@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -18,15 +18,11 @@
 import fixtures
 import mock
 
-import copy
 from nova.objects import image_meta
 from nova import test
-import pypowervm.adapter as pvm_adp
 from pypowervm import const
-import pypowervm.entities as pvm_ent
 from pypowervm.tasks import storage as tsk_stg
 from pypowervm.tests import test_fixtures as pvm_fx
-from pypowervm.tests.test_utils import pvmhttp
 from pypowervm.wrappers import cluster as pvm_clust
 from pypowervm.wrappers import storage as pvm_stg
 from pypowervm.wrappers import virtual_io_server as pvm_vios
@@ -37,13 +33,8 @@ from nova_powervm.virt.powervm.disk import ssp as ssp_dvr
 from nova_powervm.virt.powervm import exception as npvmex
 
 
-SSP = 'ssp.txt'
-CLUST = 'cluster.txt'
-VIO = 'fake_vios_with_volume_group_data.txt'
-
-
 class SSPFixture(fixtures.Fixture):
-    """Patch out PyPowerVM SSP EntryWrapper search and refresh."""
+    """Patch out PyPowerVM SSP and Cluster EntryWrapper methods."""
 
     def __init__(self):
         pass
@@ -53,12 +44,12 @@ class SSPFixture(fixtures.Fixture):
 
     def setUp(self):
         super(SSPFixture, self).setUp()
-        self.mock_search = self.mockpatch(
+        self.mock_clust_get = self.mockpatch(
+            'pypowervm.wrappers.cluster.Cluster.get')
+        self.mock_clust_search = self.mockpatch(
             'pypowervm.wrappers.cluster.Cluster.search')
-        self.mock_clust_refresh = self.mockpatch(
-            'pypowervm.wrappers.cluster.Cluster.refresh')
-        self.mock_ssp_refresh = self.mockpatch(
-            'pypowervm.wrappers.storage.SSP.refresh')
+        self.mock_ssp_gbhref = self.mockpatch(
+            'pypowervm.wrappers.storage.SSP.get_by_href')
         self.mock_ssp_update = self.mockpatch(
             'pypowervm.wrappers.storage.SSP.update')
         self.mock_get_tier = self.mockpatch(
@@ -77,35 +68,31 @@ class TestSSPDiskAdapter(test.TestCase):
 
         self.instance = Instance()
 
-        self.apt = self.useFixture(pvm_fx.AdapterFx()).adpt
-
-        def resp(file_name):
-            return pvmhttp.load_pvm_resp(
-                file_name, adapter=self.apt).get_response()
-
-        self.ssp_resp = resp(SSP)
-        self.clust_resp = resp(CLUST)
-        self.vio = resp(VIO)
+        self.apt = mock.Mock()
+        self.host_uuid = 'host_uuid'
 
         self.sspfx = self.useFixture(SSPFixture())
 
-        # For _fetch_cluster() with configured name
-        self.mock_search = self.sspfx.mock_search
-        # EntryWrapper.search always returns a list of wrappers.
-        self.mock_search.return_value = [pvm_clust.Cluster.wrap(
-            self.clust_resp)]
+        self.ssp_wrap = mock.Mock(spec=pvm_stg.SSP)
+        self.ssp_wrap.refresh.return_value = self.ssp_wrap
+        self.node1 = mock.Mock()
+        self.node2 = mock.Mock()
+        self.clust_wrap = mock.Mock(spec=pvm_clust.Cluster,
+                                    nodes=[self.node1, self.node2])
+        self.clust_wrap.refresh.return_value = self.clust_wrap
+        self.vio_wrap = mock.Mock(spec=pvm_vios.VIOS)
 
-        # For _refresh_cluster()
-        self.mock_clust_refresh = self.sspfx.mock_clust_refresh
-        self.mock_clust_refresh.return_value = pvm_clust.Cluster.wrap(
-            self.clust_resp)
+        # For _fetch_cluster() with no name
+        self.mock_clust_get = self.sspfx.mock_clust_get
+        self.mock_clust_get.return_value = [self.clust_wrap]
+        # For _fetch_cluster() with configured name
+        self.mock_clust_search = self.sspfx.mock_clust_search
+        # EntryWrapper.search always returns a list of wrappers.
+        self.mock_clust_search.return_value = [self.clust_wrap]
 
         # For _fetch_ssp() fresh
-        self.apt.read_by_href.return_value = self.ssp_resp
-
-        # For _fetch_ssp() refresh
-        self.mock_ssp_refresh = self.sspfx.mock_ssp_refresh
-        self.mock_ssp_refresh.return_value = pvm_stg.SSP.wrap(self.ssp_resp)
+        self.mock_ssp_gbhref = self.sspfx.mock_ssp_gbhref
+        self.mock_ssp_gbhref.return_value = self.ssp_wrap
 
         # For _tier
         self.mock_get_tier = self.sspfx.mock_get_tier
@@ -119,33 +106,7 @@ class TestSSPDiskAdapter(test.TestCase):
         self.mgmt_uuid.return_value = 'mp_uuid'
 
     def _get_ssp_stor(self):
-        ssp_stor = ssp_dvr.SSPDiskAdapter(
-            self.apt, '67dca605-3923-34da-bd8f-26a378fc817f')
-        return ssp_stor
-
-    def _bld_resp(self, status=200, entry_or_list=None):
-        """Build a pypowervm.adapter.Response for mocking Adapter.search/read.
-
-        :param status: HTTP status of the Response.
-        :param entry_or_list: pypowervm.entity.Entry or list thereof.  If
-                              None, the Response has no content.  If an Entry,
-                              the Response looks like it got back <entry/>.  If
-                              a list of Entry, the Response looks like it got
-                              back <feed/>.
-        :return: pypowervm.adapter.Response suitable for mocking
-                 pypowervm.adapter.Adapter.search or read.
-        """
-        resp = pvm_adp.Response('meth', 'path', status, 'reason', {})
-        resp.entry = None
-        resp.feed = None
-        if entry_or_list is None:
-            resp.feed = pvm_ent.Feed({}, [])
-        else:
-            if isinstance(entry_or_list, list):
-                resp.feed = pvm_ent.Feed({}, entry_or_list)
-            else:
-                resp.entry = entry_or_list
-        return resp
+        return ssp_dvr.SSPDiskAdapter(self.apt, self.host_uuid)
 
     def test_tier_cache(self):
         # default_tier_for_ssp not yet invoked
@@ -164,9 +125,9 @@ class TestSSPDiskAdapter(test.TestCase):
 
     def test_get_info(self):
         ssp_stor = self._get_ssp_stor()
-        expected = {'cluster_name': 'neoclust1',
-                    'ssp_name': 'neossp1',
-                    'ssp_uuid': 'e357a79a-7a3d-35b6-8405-55ab6a2d0de7'}
+        expected = {'cluster_name': self.clust_wrap.name,
+                    'ssp_name': self.ssp_wrap.name,
+                    'ssp_uuid': self.ssp_wrap.uuid}
         # Ensure the base method returns empty dict
         self.assertEqual(expected, ssp_stor.get_info())
 
@@ -188,99 +149,88 @@ class TestSSPDiskAdapter(test.TestCase):
         """
         # Invoke __init__ => _fetch_cluster()
         self._get_ssp_stor()
-        # _fetch_cluster() WITH configured name does a search, but not a read.
+        # _fetch_cluster() WITH configured name does a search, but not a get.
         # Refresh shouldn't be invoked.
-        self.assertEqual(1, self.mock_search.call_count)
-        self.assertEqual(0, self.apt.read.call_count)
-        self.assertEqual(0, self.mock_clust_refresh.call_count)
+        self.mock_clust_search.assert_called_once_with(self.apt, name='clust1')
+        self.mock_clust_get.assert_not_called()
+        self.clust_wrap.refresh.assert_not_called()
 
     def test_init_green_no_config(self):
         """No cluster name specified in config; one cluster on host - ok."""
         self.flags(cluster_name='', group='powervm')
-        self.apt.read.return_value = self._bld_resp(
-            entry_or_list=[self.clust_resp.entry])
         self._get_ssp_stor()
-        # _fetch_cluster() WITHOUT a configured name does a feed GET (read),
-        # not a search.
+        # _fetch_cluster() WITHOUT configured name does feed GET, not a search.
         # Refresh shouldn't be invoked.
-        self.assertEqual(0, self.mock_search.call_count)
-        self.assertEqual(1, self.apt.read.call_count)
-        self.assertEqual(0, self.mock_clust_refresh.call_count)
-        self.apt.read.assert_called_with('Cluster')
+        self.mock_clust_search.assert_not_called()
+        self.mock_clust_get.assert_called_once_with(self.apt)
+        self.clust_wrap.refresh.assert_not_called()
 
     def test_init_ClusterNotFoundByName(self):
         """Empty feed comes back from search - no cluster by that name."""
-        self.mock_search.return_value = []
+        self.mock_clust_search.return_value = []
         self.assertRaises(npvmex.ClusterNotFoundByName, self._get_ssp_stor)
 
     def test_init_TooManyClustersFound(self):
         """Search-by-name returns more than one result."""
-        clust1 = pvm_clust.Cluster.bld(None, 'newclust1',
-                                       pvm_stg.PV.bld(None, 'hdisk1'),
-                                       pvm_clust.Node.bld('vios1'))
-        clust2 = pvm_clust.Cluster.bld(None, 'newclust2',
-                                       pvm_stg.PV.bld(None, 'hdisk2'),
-                                       pvm_clust.Node.bld(None, 'vios2'))
-        self.mock_search.return_value = [clust1.entry, clust2.entry]
+        self.mock_clust_search.return_value = ['newclust1', 'newclust2']
         self.assertRaises(npvmex.TooManyClustersFound, self._get_ssp_stor)
 
     def test_init_NoConfigNoClusterFound(self):
         """No cluster name specified in config, no clusters on host."""
         self.flags(cluster_name='', group='powervm')
-        self.apt.read.return_value = self._bld_resp(status=204)
+        self.mock_clust_get.return_value = []
         self.assertRaises(npvmex.NoConfigNoClusterFound, self._get_ssp_stor)
 
     def test_init_NoConfigTooManyClusters(self):
         """No SSP name specified in config, more than one SSP on host."""
         self.flags(cluster_name='', group='powervm')
-        clust1 = pvm_clust.Cluster.bld(None, 'newclust1',
-                                       pvm_stg.PV.bld(None, 'hdisk1'),
-                                       pvm_clust.Node.bld(None, 'vios1'))
-        clust2 = pvm_clust.Cluster.bld(None, 'newclust2',
-                                       pvm_stg.PV.bld(None, 'hdisk2'),
-                                       pvm_clust.Node.bld(None, 'vios2'))
-        self.apt.read.return_value = self._bld_resp(
-            entry_or_list=[clust1.entry, clust2.entry])
+        self.mock_clust_get.return_value = ['newclust1', 'newclust2']
         self.assertRaises(npvmex.NoConfigTooManyClusters, self._get_ssp_stor)
 
     def test_refresh_cluster(self):
         """_refresh_cluster with cached wrapper."""
         # Save original cluster wrapper for later comparison
-        orig_clust_wrap = pvm_clust.Cluster.wrap(self.clust_resp)
+        orig_clust_wrap = self.clust_wrap
         # Prime _clust_wrap
         ssp_stor = self._get_ssp_stor()
         # Verify baseline call counts
-        self.assertEqual(1, self.mock_search.call_count)
-        self.assertEqual(0, self.mock_clust_refresh.call_count)
+        self.mock_clust_search.assert_called_once_with(self.apt, name='clust1')
+        self.clust_wrap.refresh.assert_not_called()
         clust_wrap = ssp_stor._refresh_cluster()
         # This should call refresh
-        self.assertEqual(1, self.mock_search.call_count)
-        self.assertEqual(0, self.apt.read.call_count)
-        self.assertEqual(1, self.mock_clust_refresh.call_count)
+        self.mock_clust_search.assert_called_once_with(self.apt, name='clust1')
+        self.mock_clust_get.assert_not_called()
+        self.clust_wrap.refresh.assert_called_once_with()
         self.assertEqual(clust_wrap.name, orig_clust_wrap.name)
 
     def test_fetch_ssp(self):
         # For later comparison
-        orig_ssp_wrap = pvm_stg.SSP.wrap(self.ssp_resp)
+        orig_ssp_wrap = self.ssp_wrap
         # Verify baseline call counts
-        self.assertEqual(0, self.apt.read_by_href.call_count)
-        self.assertEqual(0, self.mock_ssp_refresh.call_count)
+        self.mock_ssp_gbhref.assert_not_called()
+        self.ssp_wrap.refresh.assert_not_called()
         # This should prime self._ssp_wrap: calls read_by_href but not refresh.
         ssp_stor = self._get_ssp_stor()
-        self.assertEqual(1, self.apt.read_by_href.call_count)
-        self.assertEqual(0, self.mock_ssp_refresh.call_count)
+        self.mock_ssp_gbhref.assert_called_once_with(self.apt,
+                                                     self.clust_wrap.ssp_uri)
+        self.ssp_wrap.refresh.assert_not_called()
         # Accessing the @property will trigger refresh
         ssp_wrap = ssp_stor._ssp
-        self.assertEqual(1, self.apt.read_by_href.call_count)
-        self.assertEqual(1, self.mock_ssp_refresh.call_count)
+        self.mock_ssp_gbhref.assert_called_once_with(self.apt,
+                                                     self.clust_wrap.ssp_uri)
+        self.ssp_wrap.refresh.assert_called_once_with()
         self.assertEqual(ssp_wrap.name, orig_ssp_wrap.name)
 
-    def test_vios_uuids(self):
+    @mock.patch('pypowervm.util.get_req_path_uuid')
+    def test_vios_uuids(self, mock_rpu):
+        mock_rpu.return_value = self.host_uuid
         ssp_stor = self._get_ssp_stor()
         vios_uuids = ssp_stor.vios_uuids
-        self.assertEqual({'10B06F4B-437D-9C18-CA95-34006424120D',
-                          '6424120D-CA95-437D-9C18-10B06F4B3400'},
+        self.assertEqual({self.node1.vios_uuid, self.node2.vios_uuid},
                          set(vios_uuids))
+        mock_rpu.assert_has_calls(
+            [mock.call(node.vios_uri, preserve_case=True, root=True)
+             for node in [self.node1, self.node2]])
         s = set()
         for i in range(1000):
             u = ssp_stor._any_vios_uuid()
@@ -291,31 +241,28 @@ class TestSSPDiskAdapter(test.TestCase):
         # guaranteed to work, but the odds of failure should be infinitesimal.
         self.assertEqual(set(vios_uuids), s)
 
+        mock_rpu.reset_mock()
+
         # Test VIOSes on other nodes, which won't have uuid or url
-        with mock.patch.object(ssp_stor, '_cluster') as mock_clust:
-            def mock_node(uuid, uri):
-                node = mock.MagicMock()
-                node.vios_uuid = uuid
-                node.vios_uri = uri
-                return node
-            uri = ('https://9.1.2.3:12443/rest/api/uom/ManagedSystem/67dca605-'
-                   '3923-34da-bd8f-26a378fc817f/VirtualIOServer/10B06F4B-437D-'
-                   '9C18-CA95-34006424120D')
-            node1 = mock_node(None, uri)
-            node2 = mock_node('2', None)
-            # This mock is good and should be returned
-            node3 = mock_node('3', uri)
-            mock_clust.nodes = [node1, node2, node3]
-            self.assertEqual(['3'], ssp_stor.vios_uuids)
+        node1 = mock.Mock(vios_uuid=None, vios_uri='uri1')
+        node2 = mock.Mock(vios_uuid='2', vios_uri=None)
+        # This mock is good and should be returned
+        node3 = mock.Mock(vios_uuid='3', vios_uri='uri3')
+        self.clust_wrap.nodes = [node1, node2, node3]
+        self.assertEqual(['3'], ssp_stor.vios_uuids)
+        # get_req_path_uuid was only called on the good one
+        mock_rpu.assert_called_once_with('uri3', preserve_case=True, root=True)
 
     def test_capacity(self):
         ssp_stor = self._get_ssp_stor()
         self.mock_get_tier.return_value.refresh.return_value.capacity = 10
-        self.assertEqual(10.0, ssp_stor.capacity)
+        self.assertAlmostEqual(10.0, ssp_stor.capacity)
 
     def test_capacity_used(self):
         ssp_stor = self._get_ssp_stor()
-        self.assertEqual((49.88 - 48.98), ssp_stor.capacity_used)
+        self.ssp_wrap.capacity = 4.56
+        self.ssp_wrap.free_space = 1.23
+        self.assertAlmostEqual((4.56 - 1.23), ssp_stor.capacity_used)
 
     @mock.patch('pypowervm.tasks.cluster_ssp.get_or_upload_image_lu')
     @mock.patch('nova_powervm.virt.powervm.disk.driver.DiskAdapter.'
@@ -333,24 +280,26 @@ class TestSSPDiskAdapter(test.TestCase):
         img_meta = mock.Mock()
 
         ssp = self._get_ssp_stor()
-
         mock_crt_lu.return_value = ssp._ssp_wrap, 'lu'
+
         mock_gin.return_value = 'img_name'
         mock_vuuid.return_value = 'vios_uuid'
 
         # Default image_type
         self.assertEqual('lu', ssp.create_disk_from_image(
-            'ctx', instance, img_meta, 'disk_gb'))
+            'ctx', instance, img_meta))
         mock_goru.assert_called_once_with(
             self.mock_get_tier.return_value, mock_gin.return_value,
             mock_vuuid.return_value, mock_it2f.return_value, img_meta.size,
             upload_type=tsk_stg.UploadType.IO_STREAM)
-        mock_it2f.assert_called_once_with(mock_dl.return_value)
         mock_dl.assert_called_once_with('ctx', img_meta.id)
+        mock_it2f.assert_called_once_with(mock_dl.return_value)
         mock_gdn.assert_called_once_with(disk_dvr.DiskType.BOOT, instance)
         mock_crt_lu.assert_called_once_with(
             self.mock_get_tier.return_value, mock_gdn.return_value,
-            'disk_gb', typ=pvm_stg.LUType.DISK, clone=mock_goru.return_value)
+            instance.flavor.root_gb, typ=pvm_stg.LUType.DISK,
+            clone=mock_goru.return_value)
+
         # Reset
         mock_goru.reset_mock()
         mock_gdn.reset_mock()
@@ -360,16 +309,18 @@ class TestSSPDiskAdapter(test.TestCase):
 
         # Specified image_type
         self.assertEqual('lu', ssp.create_disk_from_image(
-            'ctx', instance, img_meta, 'disk_gb', image_type='imgtyp'))
+            'ctx', instance, img_meta, image_type='imgtyp'))
         mock_goru.assert_called_once_with(
             self.mock_get_tier.return_value, mock_gin.return_value,
             mock_vuuid.return_value, mock_it2f.return_value, img_meta.size,
             upload_type=tsk_stg.UploadType.IO_STREAM)
         mock_dl.assert_called_once_with('ctx', img_meta.id)
+        mock_it2f.assert_called_once_with(mock_dl.return_value)
         mock_gdn.assert_called_once_with('imgtyp', instance)
         mock_crt_lu.assert_called_once_with(
             self.mock_get_tier.return_value, mock_gdn.return_value,
-            'disk_gb', typ=pvm_stg.LUType.DISK, clone=mock_goru.return_value)
+            instance.flavor.root_gb, typ=pvm_stg.LUType.DISK,
+            clone=mock_goru.return_value)
 
     def test_get_image_name(self):
         """Generate image name from ImageMeta."""
@@ -408,7 +359,7 @@ class TestSSPDiskAdapter(test.TestCase):
             ssp.get_disk_ref(self.instance, disk_dvr.DiskType.BOOT))
 
     @mock.patch('nova_powervm.virt.powervm.disk.ssp.SSPDiskAdapter.'
-                'vios_uuids')
+                'vios_uuids', new_callable=mock.PropertyMock)
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.partition.get_active_vioses')
@@ -416,7 +367,7 @@ class TestSSPDiskAdapter(test.TestCase):
                           mock_build_map, mock_vio_uuids):
         # vio is a single-entry response.  Wrap it and put it in a list
         # to act as the feed for FeedTaskFx and FeedTask.
-        feed = [pvm_vios.VIOS.wrap(self.vio)]
+        feed = [self.vio_wrap]
         mock_active_vioses.return_value = feed
         ft_fx = pvm_fx.FeedTaskFx(feed)
         self.useFixture(ft_fx)
@@ -428,18 +379,16 @@ class TestSSPDiskAdapter(test.TestCase):
         # Need the driver to return the actual UUID of the VIOS in the feed,
         # to match the FeedTask.
         ssp = self._get_ssp_stor()
-        ssp.vios_uuids = [feed[0].uuid]
+        mock_vio_uuids.return_value = [self.vio_wrap.uuid]
         inst = mock.Mock(uuid=fx.FAKE_INST_UUID)
 
         # As initialized above, remove_maps returns True to trigger update.
-        ssp.connect_disk(mock.MagicMock(), inst, mock.MagicMock(),
-                         stg_ftsk=None)
-        self.assertEqual(1, mock_add_map.call_count)
-        mock_add_map.assert_called_once_with(feed[0], 'fake_map')
-        self.assertEqual(1, ft_fx.patchers['update'].mock.call_count)
+        ssp.connect_disk(inst, mock.Mock(), stg_ftsk=None)
+        mock_add_map.assert_called_once_with(self.vio_wrap, 'fake_map')
+        self.vio_wrap.update.assert_called_once_with(timeout=mock.ANY)
 
     @mock.patch('nova_powervm.virt.powervm.disk.ssp.SSPDiskAdapter.'
-                'vios_uuids')
+                'vios_uuids', new_callable=mock.PropertyMock)
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.partition.get_active_vioses')
@@ -447,7 +396,7 @@ class TestSSPDiskAdapter(test.TestCase):
                                     mock_build_map, mock_vio_uuids):
         # vio is a single-entry response.  Wrap it and put it in a list
         # to act as the feed for FeedTaskFx and FeedTask.
-        feed = [pvm_vios.VIOS.wrap(self.vio)]
+        feed = [self.vio_wrap]
         mock_active_vioses.return_value = feed
         ft_fx = pvm_fx.FeedTaskFx(feed)
         self.useFixture(ft_fx)
@@ -459,25 +408,23 @@ class TestSSPDiskAdapter(test.TestCase):
         # Need the driver to return the actual UUID of the VIOS in the feed,
         # to match the FeedTask.
         ssp = self._get_ssp_stor()
-        ssp.vios_uuids = [feed[0].uuid]
+        mock_vio_uuids.return_value = [self.vio_wrap.uuid]
         inst = mock.Mock(uuid=fx.FAKE_INST_UUID)
 
-        # As initialized above, remove_maps returns True to trigger update.
-        ssp.connect_disk(mock.MagicMock(), inst, mock.MagicMock(),
-                         stg_ftsk=None)
-        self.assertEqual(1, mock_add_map.call_count)
-        mock_add_map.assert_called_once_with(feed[0], 'fake_map')
-        self.assertEqual(0, ft_fx.patchers['update'].mock.call_count)
+        # As initialized above, add_maps returns False to skip update.
+        ssp.connect_disk(inst, mock.Mock(), stg_ftsk=None)
+        mock_add_map.assert_called_once_with(self.vio_wrap, 'fake_map')
+        self.vio_wrap.update.assert_not_called()
 
     @mock.patch('pypowervm.tasks.storage.rm_tier_storage')
     def test_delete_disks(self, mock_rm_tstor):
         sspdrv = self._get_ssp_stor()
-        sspdrv.delete_disks(None, None, ['disk1', 'disk2'])
+        sspdrv.delete_disks(['disk1', 'disk2'])
         mock_rm_tstor.assert_called_once_with(['disk1', 'disk2'],
                                               tier=sspdrv._tier)
 
     @mock.patch('nova_powervm.virt.powervm.disk.ssp.SSPDiskAdapter.'
-                'vios_uuids')
+                'vios_uuids', new_callable=mock.PropertyMock)
     @mock.patch('pypowervm.tasks.scsi_mapper.find_maps')
     @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
@@ -486,7 +433,7 @@ class TestSSPDiskAdapter(test.TestCase):
                              mock_remove_maps, mock_find_maps, mock_vio_uuids):
         # vio is a single-entry response.  Wrap it and put it in a list
         # to act as the feed for FeedTaskFx and FeedTask.
-        feed = [pvm_vios.VIOS.wrap(self.vio)]
+        feed = [self.vio_wrap]
         ft_fx = pvm_fx.FeedTaskFx(feed)
         mock_active_vioses.return_value = feed
         self.useFixture(ft_fx)
@@ -497,7 +444,7 @@ class TestSSPDiskAdapter(test.TestCase):
         # Need the driver to return the actual UUID of the VIOS in the feed,
         # to match the FeedTask.
         ssp = self._get_ssp_stor()
-        ssp.vios_uuids = [feed[0].uuid]
+        mock_vio_uuids.return_value = [self.vio_wrap.uuid]
 
         # Make the LU's to remove
         def mklu(udid):
@@ -517,11 +464,11 @@ class TestSSPDiskAdapter(test.TestCase):
         mock_find_maps.side_effect = remove_resp
 
         # As initialized above, remove_maps returns True to trigger update.
-        lu_list = ssp.disconnect_image_disk(mock.Mock(), self.instance,
-                                            stg_ftsk=None)
+        lu_list = ssp.disconnect_disk(self.instance, stg_ftsk=None)
         self.assertEqual({lu1, lu2}, set(lu_list))
-        self.assertEqual(1, mock_remove_maps.call_count)
-        self.assertEqual(1, ft_fx.patchers['update'].mock.call_count)
+        mock_remove_maps.assert_called_once_with(
+            self.vio_wrap, fx.FAKE_INST_UUID_PVM, match_func=mock.ANY)
+        self.vio_wrap.update.assert_called_once_with(timeout=mock.ANY)
 
     def test_shared_stg_calls(self):
 
@@ -549,121 +496,129 @@ class TestSSPDiskAdapter(test.TestCase):
         inst.name = 'my-instance-name'
         lpar_wrap = mock.Mock()
         lpar_wrap.id = 4
+        lu_wrap = mock.Mock(spec=pvm_stg.LU)
+        lu_wrap.configure_mock(name='boot_my_instance_name', udid='lu_udid')
+        smap = mock.Mock(backing_storage=lu_wrap,
+                         server_adapter=mock.Mock(lpar_id=4))
         # Build mock VIOS Wrappers as the returns from VIOS.wrap.
         # vios1 and vios2 will both have the mapping for client ID 4 and LU
         # named boot_my_instance_name.
-        vios1 = pvm_vios.VIOS.wrap(pvmhttp.load_pvm_resp(
-            'fake_vios_ssp_npiv.txt', adapter=self.apt).get_response())
-        vios1.scsi_mappings[3].backing_storage._name('boot_my_instance_name')
-        resp1 = self._bld_resp(entry_or_list=vios1.entry)
-        vios2 = copy.deepcopy(vios1)
-        # Change name and UUID so we can tell the difference:
-        vios2.name = 'vios2'
-        vios2.entry.properties['id'] = '7C6475B3-73A5-B9FF-4799-B23B20292951'
-        resp2 = self._bld_resp(entry_or_list=vios2.entry)
+        smaps = [mock.Mock(), mock.Mock(), mock.Mock(), smap]
+        vios1 = mock.Mock(spec=pvm_vios.VIOS)
+        vios1.configure_mock(name='vios1', uuid='uuid1', scsi_mappings=smaps)
+        vios2 = mock.Mock(spec=pvm_vios.VIOS)
+        vios2.configure_mock(name='vios2', uuid='uuid2', scsi_mappings=smaps)
         # vios3 will not have the mapping
-        vios3 = pvm_vios.VIOS.wrap(pvmhttp.load_pvm_resp(
-            'fake_vios_ssp_npiv.txt', adapter=self.apt).get_response())
-        vios3.name = 'vios3'
-        vios3.entry.properties['id'] = 'B9FF4799-B23B-2029-2951-7C6475B373A5'
-        resp3 = self._bld_resp(entry_or_list=vios3.entry)
-        return inst, lpar_wrap, resp1, resp2, resp3
+        vios3 = mock.Mock(spec=pvm_vios.VIOS)
+        vios3.configure_mock(name='vios3', uuid='uuid3',
+                             scsi_mappings=[mock.Mock(), mock.Mock()])
+        return inst, lpar_wrap, vios1, vios2, vios3
 
-    def test_instance_disk_iter(self):
-        def assert_read_calls(num):
-            self.assertEqual(num, self.apt.read.call_count)
-            self.apt.read.assert_has_calls(
-                [mock.call(pvm_vios.VIOS.schema_type, root_id=mock.ANY,
-                           xag=[const.XAG.VIO_SMAP])
-                 for i in range(num)])
+    @mock.patch('nova_powervm.virt.powervm.disk.ssp.SSPDiskAdapter.'
+                'vios_uuids', new_callable=mock.PropertyMock)
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.get')
+    def test_instance_disk_iter(self, mock_vio_get, mock_lw, mock_vio_uuids):
+        inst, lpar_wrap, vio1, vio2, vio3 = self._bld_mocks_for_instance_disk()
+        mock_lw.return_value = lpar_wrap
+        mock_vio_uuids.return_value = [1, 2]
         ssp_stor = self._get_ssp_stor()
-        inst, lpar_wrap, rsp1, rsp2, rsp3 = self._bld_mocks_for_instance_disk()
 
-        # Test with two VIOSes, both of which contain the mapping
-        self.apt.read.side_effect = [rsp1, rsp2]
-        count = 0
-        for lu, vios in ssp_stor.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
-            count += 1
-            self.assertEqual('274d7bb790666211e3bc1a00006cae8b01ac18997ab9bc23'
-                             'fb24756e9713a93f90', lu.udid)
-            self.assertEqual('vios1_181.68' if count == 1 else 'vios2',
-                             vios.name)
-        self.assertEqual(2, count)
-        assert_read_calls(2)
+        # Test with two VIOSes, both of which contain the mapping.  Force the
+        # method to get the lpar_wrap.
+        mock_vio_get.side_effect = [vio1, vio2]
+        idi = ssp_stor.instance_disk_iter(inst)
+        lu, vios = next(idi)
+        self.assertEqual('lu_udid', lu.udid)
+        self.assertEqual('vios1', vios.name)
+        mock_vio_get.assert_called_once_with(self.apt, uuid=1,
+                                             xag=[const.XAG.VIO_SMAP])
+        lu, vios = next(idi)
+        self.assertEqual('lu_udid', lu.udid)
+        self.assertEqual('vios2', vios.name)
+        mock_vio_get.assert_called_with(self.apt, uuid=2,
+                                        xag=[const.XAG.VIO_SMAP])
+        self.assertRaises(StopIteration, next, idi)
+        self.assertEqual(2, mock_vio_get.call_count)
+        mock_lw.assert_called_once_with(self.apt, inst)
 
         # Same, but prove that breaking out of the loop early avoids the second
-        # Adapter.read call
-        self.apt.reset_mock()
-        self.apt.read.side_effect = [rsp1, rsp2]
+        # get call.  Supply lpar_wrap from here on, and prove no calls to
+        # get_instance_wrapper
+        mock_vio_get.reset_mock()
+        mock_lw.reset_mock()
+        mock_vio_get.side_effect = [vio1, vio2]
         for lu, vios in ssp_stor.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
-            self.assertEqual('274d7bb790666211e3bc1a00006cae8b01ac18997ab9bc23'
-                             'fb24756e9713a93f90', lu.udid)
-            self.assertEqual('vios1_181.68', vios.name)
+            self.assertEqual('lu_udid', lu.udid)
+            self.assertEqual('vios1', vios.name)
             break
-        self.assertEqual(1, self.apt.read.call_count)
-        assert_read_calls(1)
+        mock_vio_get.assert_called_once_with(self.apt, uuid=1,
+                                             xag=[const.XAG.VIO_SMAP])
 
         # Now the first VIOS doesn't have the mapping, but the second does
-        self.apt.reset_mock()
-        self.apt.read.side_effect = [rsp3, rsp2]
-        count = 0
-        for lu, vios in ssp_stor.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
-            count += 1
-            self.assertEqual('274d7bb790666211e3bc1a00006cae8b01ac18997ab9bc23'
-                             'fb24756e9713a93f90', lu.udid)
-            self.assertEqual('vios2', vios.name)
-        self.assertEqual(1, count)
-        assert_read_calls(2)
+        mock_vio_get.reset_mock()
+        mock_vio_get.side_effect = [vio3, vio2]
+        idi = ssp_stor.instance_disk_iter(inst, lpar_wrap=lpar_wrap)
+        lu, vios = next(idi)
+        self.assertEqual('lu_udid', lu.udid)
+        self.assertEqual('vios2', vios.name)
+        mock_vio_get.assert_has_calls(
+            [mock.call(self.apt, uuid=uuid, xag=[const.XAG.VIO_SMAP])
+             for uuid in (1, 2)])
+        self.assertRaises(StopIteration, next, idi)
+        self.assertEqual(2, mock_vio_get.call_count)
 
         # No hits
-        self.apt.reset_mock()
-        self.apt.read.side_effect = [rsp3, rsp3]
-        for lu, vios in ssp_stor.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
-            self.fail()
-        assert_read_calls(2)
+        mock_vio_get.reset_mock()
+        mock_vio_get.side_effect = [vio3, vio3]
+        self.assertEqual([], list(ssp_stor.instance_disk_iter(
+            inst, lpar_wrap=lpar_wrap)))
+        self.assertEqual(2, mock_vio_get.call_count)
 
+        mock_lw.assert_not_called()
+
+    @mock.patch('nova_powervm.virt.powervm.disk.ssp.SSPDiskAdapter.'
+                'vios_uuids', new_callable=mock.PropertyMock)
     @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.get')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_vscsi_mapping')
-    def test_connect_instance_disk_to_mgmt(self, mock_add, mock_lw):
-        ssp_stor = self._get_ssp_stor()
-        inst, lpar_wrap, rsp1, rsp2, rsp3 = self._bld_mocks_for_instance_disk()
+    def test_connect_instance_disk_to_mgmt(self, mock_add, mock_vio_get,
+                                           mock_lw, mock_vio_uuids):
+        inst, lpar_wrap, vio1, vio2, vio3 = self._bld_mocks_for_instance_disk()
         mock_lw.return_value = lpar_wrap
+        mock_vio_uuids.return_value = [1, 2]
+        ssp_stor = self._get_ssp_stor()
 
         # Test with two VIOSes, both of which contain the mapping
-        self.apt.read.side_effect = [rsp1, rsp2]
+        mock_vio_get.side_effect = [vio1, vio2]
         lu, vios = ssp_stor.connect_instance_disk_to_mgmt(inst)
-        self.assertEqual('274d7bb790666211e3bc1a00006cae8b01ac18997ab9bc23'
-                         'fb24756e9713a93f90', lu.udid)
+        self.assertEqual('lu_udid', lu.udid)
         # Should hit on the first VIOS
-        self.assertIs(rsp1.entry, vios.entry)
-        self.assertEqual(1, mock_add.call_count)
-        mock_add.assert_called_with('67dca605-3923-34da-bd8f-26a378fc817f',
-                                    vios, 'mp_uuid', lu)
+        self.assertIs(vio1, vios)
+        mock_add.assert_called_once_with(self.host_uuid, vio1, 'mp_uuid', lu)
 
         # Now the first VIOS doesn't have the mapping, but the second does
         mock_add.reset_mock()
-        self.apt.read.side_effect = [rsp3, rsp2]
+        mock_vio_get.side_effect = [vio3, vio2]
         lu, vios = ssp_stor.connect_instance_disk_to_mgmt(inst)
-        self.assertEqual('274d7bb790666211e3bc1a00006cae8b01ac18997ab9bc23'
-                         'fb24756e9713a93f90', lu.udid)
+        self.assertEqual('lu_udid', lu.udid)
         # Should hit on the second VIOS
-        self.assertIs(rsp2.entry, vios.entry)
+        self.assertIs(vio2, vios)
         self.assertEqual(1, mock_add.call_count)
-        mock_add.assert_called_with('67dca605-3923-34da-bd8f-26a378fc817f',
-                                    vios, 'mp_uuid', lu)
+        mock_add.assert_called_once_with(self.host_uuid, vio2, 'mp_uuid', lu)
 
         # No hits
         mock_add.reset_mock()
-        self.apt.read.side_effect = [rsp3, rsp3]
+        mock_vio_get.side_effect = [vio3, vio3]
         self.assertRaises(npvmex.InstanceDiskMappingFailed,
                           ssp_stor.connect_instance_disk_to_mgmt, inst)
-        self.assertEqual(0, mock_add.call_count)
+        mock_add.assert_not_called()
 
         # First add_vscsi_mapping call raises
-        self.apt.read.side_effect = [rsp1, rsp2]
+        mock_vio_get.side_effect = [vio1, vio2]
         mock_add.side_effect = [Exception("mapping failed"), None]
         # Should hit on the second VIOS
-        self.assertIs(rsp2.entry, vios.entry)
+        self.assertIs(vio2, vios)
 
     @mock.patch('pypowervm.tasks.scsi_mapper.remove_lu_mapping')
     def test_disconnect_disk_from_mgmt(self, mock_rm_lu_map):
