@@ -37,6 +37,7 @@ from nova_powervm.tests.virt import powervm
 from nova_powervm.virt.powervm import exception as nvex
 from nova_powervm.virt.powervm import vm
 
+
 LPAR_HTTPRESP_FILE = "lpar.txt"
 LPAR_MAPPING = (
     {
@@ -88,8 +89,7 @@ class TestVMBuilder(test.TestCase):
         flavor.extra_specs = {'powervm:dedicated_proc': 'true'}
         test_attrs = dict(lpar_attrs, dedicated_proc='true')
 
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
@@ -103,32 +103,28 @@ class TestVMBuilder(test.TestCase):
                           dedicated_proc='true',
                           sharing_mode='sre idle procs active',
                           min_vcpu='1', max_vcpu='3')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
         # Test shared proc sharing mode
         flavor.extra_specs = {'powervm:uncapped': 'true'}
         test_attrs = dict(lpar_attrs, sharing_mode='uncapped')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
         # Test availability priority
         flavor.extra_specs = {'powervm:availability_priority': '150'}
         test_attrs = dict(lpar_attrs, avail_priority='150')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
         # Test processor compatibility
         flavor.extra_specs = {'powervm:processor_compatibility': 'POWER8'}
         test_attrs = dict(lpar_attrs, processor_compatibility='POWER8')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
@@ -136,8 +132,7 @@ class TestVMBuilder(test.TestCase):
         test_attrs = dict(
             lpar_attrs,
             processor_compatibility=pvm_bp.LPARCompat.POWER6_PLUS)
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
@@ -146,8 +141,7 @@ class TestVMBuilder(test.TestCase):
         test_attrs = dict(
             lpar_attrs,
             processor_compatibility=pvm_bp.LPARCompat.POWER6_PLUS_ENHANCED)
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
@@ -156,8 +150,7 @@ class TestVMBuilder(test.TestCase):
                               'powervm:max_proc_units': '2.0'}
         test_attrs = dict(lpar_attrs, min_proc_units='0.5',
                           max_proc_units='2.0')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
@@ -165,16 +158,14 @@ class TestVMBuilder(test.TestCase):
         flavor.extra_specs = {'powervm:min_mem': '1024',
                               'powervm:max_mem': '4096'}
         test_attrs = dict(lpar_attrs, min_mem='1024', max_mem='4096')
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
         self.san_lpar_name.assert_called_with(instance.name)
         self.san_lpar_name.reset_mock()
 
         # Test remote restart set to false
         flavor.extra_specs = {'powervm:srr_capability': 'false'}
         test_attrs = dict(lpar_attrs, srr_capability=False)
-        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
-                         test_attrs)
+        self.assertEqual(self.lpar_b._format_flavor(instance), test_attrs)
 
     @mock.patch('pypowervm.wrappers.shared_proc_pool.SharedProcPool.search')
     def test_spp_pool_id(self, mock_search):
@@ -300,8 +291,8 @@ class TestVM(test.TestCase):
         self.assertEqual(inst_info.cpu_time_ns, 0)
 
         # Check that we raise an exception if the instance is gone.
-        exc = pvm_exc.Error('Not found', response=FakeAdapterResponse(404))
-        self.apt.read.side_effect = exc
+        self.apt.read.side_effect = pvm_exc.HttpNotFound(
+            mock.MagicMock(status=404))
         self.assertRaises(exception.InstanceNotFound,
                           inst_info.__getattribute__, 'state')
 
@@ -312,12 +303,8 @@ class TestVM(test.TestCase):
             def __init__(self, body):
                 self.body = '"%s"' % body
 
-        resp = FakeResp2('running')
-
-        def return_resp(*args, **kwds):
-            return resp
-
-        self.apt.read.side_effect = return_resp
+        self.apt.read.side_effect = None
+        self.apt.read.return_value = FakeResp2('running')
         self.assertEqual(inst_info.state, power_state.RUNNING)
 
         # Check the __eq__ method
@@ -345,28 +332,85 @@ class TestVM(test.TestCase):
         self.assertEqual(lpar_list[0], 'z3-9-5-126-208-000001f0')
         self.assertEqual(len(lpar_list), 20)
 
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
     @mock.patch('pypowervm.tasks.vterm.close_vterm')
-    def test_dlt_lpar(self, mock_vterm):
+    def test_dlt_lpar(self, mock_vterm, mock_pvm_uuid):
         """Performs a delete LPAR test."""
-        vm.dlt_lpar(self.apt, '12345')
-        self.assertEqual(1, self.apt.delete.call_count)
-        self.assertEqual(1, mock_vterm.call_count)
+        mock_pvm_uuid.return_value = 'pvm_uuid'
+
+        vm.delete_lpar(self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_called_once_with('LogicalPartition',
+                                                root_id='pvm_uuid')
 
         # Test Failure Path
         # build a mock response body with the expected HSCL msg
-        resp = mock.Mock()
-        resp.body = 'error msg: HSCL151B more text'
+        resp = mock.Mock(body='error msg: HSCL151B more text')
         self.apt.delete.side_effect = pvm_exc.Error(
             'Mock Error Message', response=resp)
 
         # Reset counters
+        mock_pvm_uuid.reset_mock()
         self.apt.reset_mock()
         mock_vterm.reset_mock()
 
         self.assertRaises(pvm_exc.Error,
-                          vm.dlt_lpar, self.apt, '12345')
-        self.assertEqual(1, mock_vterm.call_count)
-        self.assertEqual(1, self.apt.delete.call_count)
+                          vm.delete_lpar, self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_called_once_with('LogicalPartition',
+                                                root_id='pvm_uuid')
+
+        # Test HttpNotFound - exception not raised
+        mock_pvm_uuid.reset_mock()
+        self.apt.reset_mock()
+        mock_vterm.reset_mock()
+
+        resp.status = 404
+        self.apt.delete.side_effect = pvm_exc.HttpNotFound(resp=resp)
+        vm.delete_lpar(self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_called_once_with('LogicalPartition',
+                                                root_id='pvm_uuid')
+
+        # Test Other HttpError
+        mock_pvm_uuid.reset_mock()
+        self.apt.reset_mock()
+        mock_vterm.reset_mock()
+
+        resp.status = 111
+        self.apt.delete.side_effect = pvm_exc.HttpError(resp=resp)
+        self.assertRaises(pvm_exc.HttpError, vm.delete_lpar, self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_called_once_with('LogicalPartition',
+                                                root_id='pvm_uuid')
+
+        # Test HttpNotFound closing vterm
+        mock_pvm_uuid.reset_mock()
+        self.apt.reset_mock()
+        mock_vterm.reset_mock()
+
+        resp.status = 404
+        mock_vterm.side_effect = pvm_exc.HttpNotFound(resp=resp)
+        vm.delete_lpar(self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_not_called()
+
+        # Test Other HttpError closing vterm
+        mock_pvm_uuid.reset_mock()
+        self.apt.reset_mock()
+        mock_vterm.reset_mock()
+
+        resp.status = 111
+        mock_vterm.side_effect = pvm_exc.HttpError(resp=resp)
+        self.assertRaises(pvm_exc.HttpError, vm.delete_lpar, self.apt, 'inst')
+        mock_pvm_uuid.assert_called_once_with('inst')
+        mock_vterm.assert_called_once_with(self.apt, 'pvm_uuid')
+        self.apt.delete.assert_not_called()
 
     @mock.patch('nova_powervm.virt.powervm.vm.VMBuilder._add_IBMi_attrs')
     @mock.patch('pypowervm.utils.lpar_builder.DefaultStandardize')
@@ -381,46 +425,57 @@ class TestVM(test.TestCase):
         lparw = pvm_lpar.LPAR.wrap(self.resp.feed.entries[0])
         mock_bld.return_value = lparw
         self.apt.create.return_value = lparw.entry
-        vm.crt_lpar(self.apt, host_wrapper, instance, flavor, nvram='data')
+        vm.create_lpar(self.apt, host_wrapper, instance, nvram='data')
         self.apt.create.assert_called_once_with(
-            lparw, 'ManagedSystem', child_type='LogicalPartition',
+            lparw, host_wrapper.schema_type, child_type='LogicalPartition',
             root_id=host_wrapper.uuid, service='uom', timeout=-1)
+        mock_stdz.assert_called_once_with(host_wrapper, uncapped_weight=64,
+                                          proc_units_factor=0.1)
         self.assertEqual(lparw.nvram, 'data')
         self.assertTrue(mock_vld_all.called)
 
-        # Test srr
+        # Test srr and slot_mgr
         self.apt.reset_mock()
         mock_vld_all.reset_mock()
+        mock_stdz.reset_mock()
         flavor.extra_specs = {'powervm:srr_capability': 'true'}
         self.apt.create.return_value = lparw.entry
-        vm.crt_lpar(self.apt, host_wrapper, instance, flavor)
+        mock_slot_mgr = mock.Mock(build_map=mock.Mock(
+            get_max_vslots=mock.Mock(return_value=123)))
+        vm.create_lpar(self.apt, host_wrapper, instance,
+                       slot_mgr=mock_slot_mgr)
         self.assertTrue(self.apt.create.called)
         self.assertTrue(mock_vld_all.called)
         self.assertTrue(lparw.srr_enabled)
+        mock_stdz.assert_called_once_with(host_wrapper, uncapped_weight=64,
+                                          proc_units_factor=0.1, max_slots=123)
+        # The save is called with the LPAR's actual value, which in this mock
+        # setup comes from lparw
+        mock_slot_mgr.register_max_vslots.assert_called_with(
+            lparw.io_config.max_virtual_slots)
 
         # Test to verify the LPAR Creation with invalid name specification
         mock_bld.side_effect = lpar_bld.LPARBuilderException("Invalid Name")
         host_wrapper = mock.Mock()
-        self.assertRaises(exception.BuildAbortException, vm.crt_lpar,
-                          self.apt, host_wrapper, instance, flavor)
+        self.assertRaises(exception.BuildAbortException, vm.create_lpar,
+                          self.apt, host_wrapper, instance)
 
         resp = mock.Mock(status=202, method='fake', path='/dev/',
                          reason='Failure')
         mock_bld.side_effect = pvm_exc.HttpError(resp)
         try:
-            vm.crt_lpar(self.apt, host_wrapper, instance, flavor)
+            vm.create_lpar(self.apt, host_wrapper, instance)
         except nvex.PowerVMAPIFailed as e:
             self.assertEqual(e.kwargs['inst_name'], instance.name)
             self.assertEqual(e.kwargs['reason'], mock_bld.side_effect)
         flavor.extra_specs = {'powervm:BADATTR': 'true'}
         host_wrapper = mock.Mock()
-        self.assertRaises(exception.InvalidAttribute, vm.crt_lpar,
-                          self.apt, host_wrapper, instance, flavor)
+        self.assertRaises(exception.InvalidAttribute, vm.create_lpar,
+                          self.apt, host_wrapper, instance)
 
     @mock.patch('pypowervm.wrappers.logical_partition.LPAR.get')
     def test_get_instance_wrapper(self, mock_get):
-        resp = mock.Mock(status=404)
-        mock_get.side_effect = pvm_exc.Error('message', response=resp)
+        mock_get.side_effect = pvm_exc.HttpNotFound(resp=mock.Mock(status=404))
         instance = objects.Instance(**powervm.TEST_INSTANCE)
         # vm.get_instance_wrapper(self.apt, instance, 'lpar_uuid')
         self.assertRaises(exception.InstanceNotFound, vm.get_instance_wrapper,
@@ -430,12 +485,12 @@ class TestVM(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.vm.VMBuilder')
     def test_update(self, mock_vmb, mock_get_inst):
         instance = objects.Instance(**powervm.TEST_INSTANCE)
-        flavor, entry = mock.Mock(), mock.Mock()
+        entry = mock.Mock()
         name = "new_name"
         entry.update.return_value = 'NewEntry'
         bldr = mock_vmb.return_value
         lpar_bldr = bldr.lpar_builder.return_value
-        new_entry = vm.update(self.apt, 'mock_host_wrap', instance, flavor,
+        new_entry = vm.update(self.apt, 'mock_host_wrap', instance,
                               entry=entry, name=name)
         # Ensure the lpar was rebuilt
         lpar_bldr.rebuild.assert_called_once_with(entry)
@@ -453,8 +508,7 @@ class TestVM(test.TestCase):
 
         entry = mock.Mock()
         entry.update.return_value = 'NewEntry'
-        new_entry = vm.rename(self.apt, 'mock_host_uuid', instance, 'new_name',
-                              entry=entry)
+        new_entry = vm.rename(self.apt, instance, 'new_name', entry=entry)
         self.assertEqual('new_name', entry.name)
         entry.update.assert_called_once_with()
         mock_entry_transaction.assert_called_once_with(mock.ANY)
@@ -465,9 +519,8 @@ class TestVM(test.TestCase):
         # Test optional entry parameter
         entry.reset_mock()
         mock_get_inst.return_value = entry
-        new_entry = vm.rename(self.apt, 'mock_host_uuid', instance, 'new_name')
-        mock_get_inst.assert_called_once_with(self.apt, instance,
-                                              'mock_host_uuid')
+        new_entry = vm.rename(self.apt, instance, 'new_name')
+        mock_get_inst.assert_called_once_with(self.apt, instance)
         self.assertEqual('new_name', entry.name)
         entry.update.assert_called_once_with()
         self.assertEqual('NewEntry', new_entry)
@@ -491,14 +544,47 @@ class TestVM(test.TestCase):
         bldr._add_IBMi_attrs(inst, attrs)
         self.assertDictEqual(attrs, {'env': 'OS400'})
 
-    @mock.patch('pypowervm.tasks.power.power_off')
-    def test_power_off(self, mock_power_off):
+    @mock.patch('pypowervm.tasks.power.power_on')
+    @mock.patch('oslo_concurrency.lockutils.lock')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    def test_power_on(self, mock_wrap, mock_lock, mock_power_on):
         instance = objects.Instance(**powervm.TEST_INSTANCE)
+        entry = mock.Mock(state=pvm_bp.LPARState.NOT_ACTIVATED)
+        mock_wrap.return_value = entry
 
-        self.assertFalse(vm.power_off(
-            None, instance, 'host_uuid',
-            mock.Mock(state=pvm_bp.LPARState.NOT_ACTIVATED)))
-        self.assertFalse(mock_power_off.called)
+        self.assertTrue(vm.power_on(None, instance, opts='opts'))
+        mock_power_on.assert_called_once_with(entry, None, add_parms='opts')
+        mock_lock.assert_called_once_with('power_%s' % instance.uuid)
+
+        mock_power_on.reset_mock()
+        mock_lock.reset_mock()
+
+        stop_states = [
+            pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
+            pvm_bp.LPARState.OPEN_FIRMWARE, pvm_bp.LPARState.SHUTTING_DOWN,
+            pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING,
+            pvm_bp.LPARState.SUSPENDING]
+
+        for stop_state in stop_states:
+            entry.state = stop_state
+            self.assertFalse(vm.power_on(None, instance))
+            mock_power_on.assert_not_called()
+            mock_lock.assert_called_once_with('power_%s' % instance.uuid)
+            mock_lock.reset_mock()
+
+    @mock.patch('pypowervm.tasks.power.PowerOp', autospec=True)
+    @mock.patch('pypowervm.tasks.power.power_off_progressive', autospec=True)
+    @mock.patch('oslo_concurrency.lockutils.lock')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    def test_power_off(self, mock_wrap, mock_lock, mock_power_off, mock_pop):
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
+        entry = mock.Mock(state=pvm_bp.LPARState.NOT_ACTIVATED)
+        mock_wrap.return_value = entry
+
+        self.assertFalse(vm.power_off(None, instance))
+        self.assertEqual(0, mock_power_off.call_count)
+        self.assertEqual(0, mock_pop.stop.call_count)
+        mock_lock.assert_called_once_with('power_%s' % instance.uuid)
 
         stop_states = [
             pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
@@ -506,38 +592,100 @@ class TestVM(test.TestCase):
             pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING,
             pvm_bp.LPARState.SUSPENDING]
         for stop_state in stop_states:
-            entry = mock.Mock(state=stop_state)
+            entry.state = stop_state
             mock_power_off.reset_mock()
-            self.assertTrue(vm.power_off(None, instance, 'host_uuid', entry))
-            mock_power_off.assert_called_once_with(entry, 'host_uuid',
-                                                   force_immediate=False,
-                                                   add_parms=None)
+            mock_pop.stop.reset_mock()
+            mock_lock.reset_mock()
+            self.assertTrue(vm.power_off(None, instance))
+            mock_power_off.assert_called_once_with(entry)
+            self.assertEqual(0, mock_pop.stop.call_count)
+            mock_lock.assert_called_once_with('power_%s' % instance.uuid)
             mock_power_off.reset_mock()
-            self.assertTrue(vm.power_off(None, instance, 'host_uuid', entry,
-                                         force_immediate=True))
-            mock_power_off.assert_called_once_with(entry, 'host_uuid',
-                                                   force_immediate=True,
-                                                   add_parms=None)
+            mock_lock.reset_mock()
+            self.assertTrue(vm.power_off(
+                None, instance, force_immediate=True, timeout=5))
+            self.assertEqual(0, mock_power_off.call_count)
+            mock_pop.stop.assert_called_once_with(
+                entry, opts=mock.ANY, timeout=5)
+            self.assertEqual('PowerOff(immediate=true, operation=shutdown)',
+                             str(mock_pop.stop.call_args[1]['opts']))
+            mock_lock.assert_called_once_with('power_%s' % instance.uuid)
 
-    @mock.patch('pypowervm.tasks.power.power_off')
-    def test_power_off_negative(self, mock_power_off):
+    @mock.patch('pypowervm.tasks.power.power_off_progressive', autospec=True)
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    def test_power_off_negative(self, mock_wrap, mock_power_off):
         """Negative tests."""
         instance = objects.Instance(**powervm.TEST_INSTANCE)
+        mock_wrap.return_value = mock.Mock(state=pvm_bp.LPARState.RUNNING)
 
         # Raise the expected pypowervm exception
         mock_power_off.side_effect = pvm_exc.VMPowerOffFailure(
             reason='Something bad.', lpar_nm='TheLPAR')
         # We should get a valid Nova exception that the compute manager expects
         self.assertRaises(exception.InstancePowerOffFailure,
-                          vm.power_off, None, instance, 'host_uuid',
-                          mock.Mock(state=pvm_bp.LPARState.RUNNING))
+                          vm.power_off, None, instance)
+
+    @mock.patch('oslo_concurrency.lockutils.lock')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    @mock.patch('pypowervm.tasks.power.power_on', autospec=True)
+    @mock.patch('pypowervm.tasks.power.power_off_progressive', autospec=True)
+    @mock.patch('pypowervm.tasks.power.PowerOp', autospec=True)
+    def test_reboot(self, mock_pop, mock_pwroff, mock_pwron, mock_giw,
+                    mock_lock):
+        entry = mock.Mock()
+        inst = mock.Mock(uuid='uuid')
+        mock_giw.return_value = entry
+
+        # VM is in 'not activated' state
+        entry.state = pvm_bp.LPARState.NOT_ACTIVATED
+        vm.reboot('adapter', inst, True)
+        mock_pwron.assert_called_once_with(entry, None)
+        self.assertEqual(0, mock_pwroff.call_count)
+        self.assertEqual(0, mock_pop.stop.call_count)
+        mock_lock.assert_called_once_with('power_uuid')
+
+        mock_pwron.reset_mock()
+        mock_lock.reset_mock()
+
+        # VM is in an active state
+        entry.state = pvm_bp.LPARState.RUNNING
+        vm.reboot('adapter', inst, True)
+        self.assertEqual(0, mock_pwron.call_count)
+        self.assertEqual(0, mock_pwroff.call_count)
+        mock_pop.stop.assert_called_once_with(entry, opts=mock.ANY)
+        self.assertEqual(
+            'PowerOff(immediate=true, operation=shutdown, restart=true)',
+            str(mock_pop.stop.call_args[1]['opts']))
+        mock_lock.assert_called_once_with('power_uuid')
+
+        mock_pop.stop.reset_mock()
+        mock_lock.reset_mock()
+
+        # Same, but soft
+        vm.reboot('adapter', inst, False)
+        self.assertEqual(0, mock_pwron.call_count)
+        mock_pwroff.assert_called_once_with(entry, restart=True)
+        self.assertEqual(0, mock_pop.stop.call_count)
+        mock_lock.assert_called_once_with('power_uuid')
+
+        mock_pwroff.reset_mock()
+        mock_lock.reset_mock()
+
+        # Exception path
+        mock_pwroff.side_effect = Exception()
+        self.assertRaises(exception.InstanceRebootFailure, vm.reboot,
+                          'adapter', inst, False)
+        self.assertEqual(0, mock_pwron.call_count)
+        mock_pwroff.assert_called_once_with(entry, restart=True)
+        self.assertEqual(0, mock_pop.stop.call_count)
+        mock_lock.assert_called_once_with('power_uuid')
 
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_qp')
     def test_instance_exists(self, mock_getvmqp, mock_getuuid):
         # Try the good case where it exists
         mock_getvmqp.side_effect = 'fake_state'
-        mock_parms = (mock.Mock(), mock.Mock(), mock.Mock())
+        mock_parms = (mock.Mock(), mock.Mock())
         self.assertTrue(vm.instance_exists(*mock_parms))
 
         # Test the scenario where it does not exist.
@@ -605,13 +753,16 @@ class TestVM(test.TestCase):
 
         resp = mock.MagicMock()
         resp.status = 404
-        self.apt.read.side_effect = pvm_exc.Error('message', response=resp)
+        self.apt.read.side_effect = pvm_exc.HttpNotFound(resp)
         self.assertRaises(exception.InstanceNotFound, vm.get_vm_qp, self.apt,
                           'lpar_uuid', log_errors=False)
 
-        resp.status = 500
+        self.apt.read.side_effect = pvm_exc.Error("message", response=None)
+        self.assertRaises(pvm_exc.Error, vm.get_vm_qp, self.apt,
+                          'lpar_uuid', log_errors=False)
 
-        self.apt.read.side_effect = pvm_exc.Error('message', response=resp)
+        resp.status = 500
+        self.apt.read.side_effect = pvm_exc.Error("message", response=resp)
         self.assertRaises(pvm_exc.Error, vm.get_vm_qp, self.apt,
                           'lpar_uuid', log_errors=False)
 
@@ -626,16 +777,56 @@ class TestVM(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
     def test_update_ibmi_settings(self, mock_lparw, mock_ibmi):
         instance = mock.MagicMock()
+
         # Test update load source with vscsi boot
         boot_type = 'vscsi'
-        vm.update_ibmi_settings(
-            self.apt, instance, 'host-uuid', boot_type)
-        mock_ibmi.assert_called_once_with(
-            self.apt, mock.ANY, 'vscsi')
+        vm.update_ibmi_settings(self.apt, instance, boot_type)
+        mock_ibmi.assert_called_once_with(self.apt, mock.ANY, 'vscsi')
         mock_ibmi.reset_mock()
+
         # Test update load source with npiv boot
         boot_type = 'npiv'
-        vm.update_ibmi_settings(
-            self.apt, instance, 'host-uuid', boot_type)
-        mock_ibmi.assert_called_once_with(
-            self.apt, mock.ANY, 'npiv')
+        vm.update_ibmi_settings(self.apt, instance, boot_type)
+        mock_ibmi.assert_called_once_with(self.apt, mock.ANY, 'npiv')
+
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.wrappers.network.CNA.search')
+    @mock.patch('pypowervm.wrappers.network.CNA.get')
+    def test_get_cnas(self, mock_get, mock_search, mock_uuid):
+        # No kwargs: get
+        self.assertEqual(mock_get.return_value, vm.get_cnas(self.apt, 'inst'))
+        mock_uuid.assert_called_once_with('inst')
+        mock_get.assert_called_once_with(self.apt, parent_type=pvm_lpar.LPAR,
+                                         parent_uuid=mock_uuid.return_value)
+        mock_search.assert_not_called()
+        # With kwargs: search
+        mock_get.reset_mock()
+        mock_uuid.reset_mock()
+        self.assertEqual(mock_search.return_value, vm.get_cnas(
+            self.apt, 'inst', one=2, three=4))
+        mock_uuid.assert_called_once_with('inst')
+        mock_search.assert_called_once_with(
+            self.apt, parent_type=pvm_lpar.LPAR,
+            parent_uuid=mock_uuid.return_value, one=2, three=4)
+        mock_get.assert_not_called()
+
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.wrappers.iocard.VNIC.search')
+    @mock.patch('pypowervm.wrappers.iocard.VNIC.get')
+    def test_get_vnics(self, mock_get, mock_search, mock_uuid):
+        # No kwargs: get
+        self.assertEqual(mock_get.return_value, vm.get_vnics(self.apt, 'inst'))
+        mock_uuid.assert_called_once_with('inst')
+        mock_get.assert_called_once_with(self.apt, parent_type=pvm_lpar.LPAR,
+                                         parent_uuid=mock_uuid.return_value)
+        mock_search.assert_not_called()
+        # With kwargs: search
+        mock_get.reset_mock()
+        mock_uuid.reset_mock()
+        self.assertEqual(mock_search.return_value, vm.get_vnics(
+            self.apt, 'inst', one=2, three=4))
+        mock_uuid.assert_called_once_with('inst')
+        mock_search.assert_called_once_with(
+            self.apt, parent_type=pvm_lpar.LPAR,
+            parent_uuid=mock_uuid.return_value, one=2, three=4)
+        mock_get.assert_not_called()
