@@ -1,5 +1,5 @@
 # Copyright 2013 OpenStack Foundation
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -15,24 +15,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_concurrency import lockutils
 from oslo_log import log as logging
+from oslo_utils import excutils
 
 from nova import exception as nova_exc
 from nova import image
+
 from pypowervm import const as pvm_const
 from pypowervm import exceptions as pvm_exc
 from pypowervm.tasks import partition as pvm_tpar
 from pypowervm.tasks import scsi_mapper as tsk_map
 from pypowervm.tasks import storage as tsk_stg
-from pypowervm.wrappers import managed_system as pvm_ms
 from pypowervm.wrappers import storage as pvm_stg
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm.disk import driver as disk_dvr
+from nova_powervm.virt.powervm.disk import imagecache
 from nova_powervm.virt.powervm import exception as npvmex
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
+from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm import vm
 
 
@@ -42,13 +44,28 @@ IMAGE_API = image.API()
 
 
 class LocalStorage(disk_dvr.DiskAdapter):
+
+    capabilities = {
+        'shared_storage': False,
+        'has_imagecache': True,
+    }
+
     def __init__(self, adapter, host_uuid):
         super(LocalStorage, self).__init__(adapter, host_uuid)
 
         # Query to get the Volume Group UUID
+        if not CONF.powervm.volume_group_name:
+            raise npvmex.OptRequiredIfOtherOptValue(
+                if_opt='disk_driver', if_value='localdisk',
+                then_opt='volume_group_name')
         self.vg_name = CONF.powervm.volume_group_name
-        self._vios_uuid, self.vg_uuid = self._get_vg_uuid(self.vg_name)
-        LOG.info(_LI("Local Storage driver initialized: volume group: '%s'"),
+        vios_w, vg_w = tsk_stg.find_vg(adapter, self.vg_name)
+        self._vios_uuid = vios_w.uuid
+        self.vg_uuid = vg_w.uuid
+        self.image_cache_mgr = imagecache.ImageManager(self._vios_uuid,
+                                                       self.vg_uuid, adapter)
+        self.cache_lock = lockutils.ReaderWriterLock()
+        LOG.info("Local Storage driver initialized: volume group: '%s'",
                  self.vg_name)
 
     @property
@@ -91,24 +108,29 @@ class LocalStorage(disk_dvr.DiskAdapter):
         # Subtract available from capacity
         return float(vg_wrap.capacity) - float(vg_wrap.available_size)
 
-    def delete_disks(self, context, instance, storage_elems):
+    def manage_image_cache(self, context, all_instances):
+        """Update the image cache
+
+        :param context: nova context
+        :param all_instances: List of all instances on the node
+        """
+        with self.cache_lock.write_lock():
+            self.image_cache_mgr.update(context, all_instances)
+
+    def delete_disks(self, storage_elems):
         """Removes the specified disks.
 
-        :param context: nova context for operation
-        :param instance: instance to delete the disk for.
         :param storage_elems: A list of the storage elements that are to be
                               deleted.  Derived from the return value from
-                              disconnect_image_disk.
+                              disconnect_disk.
         """
         # All of local disk is done against the volume group.  So reload
         # that (to get new etag) and then update against it.
         tsk_stg.rm_vg_storage(self._get_vg_wrap(), vdisks=storage_elems)
 
-    def disconnect_image_disk(self, context, instance, stg_ftsk=None,
-                              disk_type=None):
+    def disconnect_disk(self, instance, stg_ftsk=None, disk_type=None):
         """Disconnects the storage adapters from the image disk.
 
-        :param context: nova context for operation
         :param instance: instance to disconnect the image for.
         :param stg_ftsk: (Optional) The pypowervm transaction FeedTask for the
                          I/O Operations.  If provided, the Virtual I/O Server
@@ -133,8 +155,8 @@ class LocalStorage(disk_dvr.DiskAdapter):
 
         # Make sure the remove function will run within the transaction manager
         def rm_func(vios_w):
-            LOG.info(_LI("Disconnecting instance %(inst)s from storage disks.")
-                     % {'inst': instance.name})
+            LOG.info("Disconnecting instance from storage disks.",
+                     instance=instance)
             return tsk_map.remove_maps(vios_w, lpar_uuid,
                                        match_func=match_func)
 
@@ -162,13 +184,12 @@ class LocalStorage(disk_dvr.DiskAdapter):
         """
         tsk_map.remove_vdisk_mapping(self.adapter, vios_uuid, self.mp_uuid,
                                      disk_names=[disk_name])
-        LOG.info(_LI(
-            "Unmapped boot disk %(disk_name)s from the management partition "
-            "from Virtual I/O Server %(vios_name)s."), {
-                'disk_name': disk_name, 'mp_uuid': self.mp_uuid,
-                'vios_name': vios_uuid})
+        LOG.info("Unmapped boot disk %(disk_name)s from the management "
+                 "partition from Virtual I/O Server %(vios_name)s.",
+                 {'disk_name': disk_name, 'mp_uuid': self.mp_uuid,
+                  'vios_name': vios_uuid})
 
-    def _create_disk_from_image(self, context, instance, image_meta, disk_size,
+    def _create_disk_from_image(self, context, instance, image_meta,
                                 image_type=disk_dvr.DiskType.BOOT):
         """Creates a disk and copies the specified image to it.
 
@@ -178,38 +199,57 @@ class LocalStorage(disk_dvr.DiskAdapter):
         :param instance: instance to create the disk for.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
-        :param disk_size: The size of the disk to create in GB.  If smaller
-                          than the image, it will be ignored (as the disk
-                          must be at least as big as the image).  Must be an
-                          int.
         :param image_type: the image type. See disk constants above.
         :return: The backing pypowervm storage object that was created.
         """
-        LOG.info(_LI('Create disk.'))
+        LOG.info('Create disk.', instance=instance)
 
-        # Transfer the image
+        # Disk size to API is in bytes.  Input from flavor is in Gb
+        disk_bytes = self._disk_gb_to_bytes(instance.flavor.root_gb,
+                                            floor=image_meta.size)
         vol_name = self._get_disk_name(image_type, instance, short=True)
 
-        # Disk size to API is in bytes.  Input from method is in Gb
-        disk_bytes = self._disk_gb_to_bytes(disk_size, floor=image_meta.size)
+        with self.cache_lock.read_lock():
+            img_udid = self._get_or_upload_image(context, image_meta)
+            # Transfer the image
+            return tsk_stg.crt_copy_vdisk(
+                self.adapter, self._vios_uuid, self.vg_uuid, img_udid,
+                image_meta.size, vol_name, disk_bytes)
 
-        def upload(path):
-            IMAGE_API.download(context, image_meta.id, dest_path=path)
+    def _get_or_upload_image(self, context, image_meta):
+        """Return a cached image name
 
-        # This method will create a new disk at our specified size.  It will
-        # then put the image in the disk.  If the disk is bigger, user can
-        # resize the disk, create a new partition, etc...
-        # If the image is bigger than disk, API should make the disk big
-        # enough to support the image (up to 1 Gb boundary).
-        return tsk_stg.upload_new_vdisk(
-            self.adapter, self._vios_uuid, self.vg_uuid, upload, vol_name,
-            image_meta.size, d_size=disk_bytes,
-            upload_type=tsk_stg.UploadType.FUNC)[0]
+        Attempt to find a cached copy of the image. If there is no cached copy
+        of the image, create one.
 
-    def connect_disk(self, context, instance, disk_info, stg_ftsk=None):
+        :param context: nova context used to retrieve image from glance
+        :param nova.objects.ImageMeta image_meta:
+            The metadata of the image of the instance.
+        :return: The name of the virtual disk containing the image
+        """
+
+        # Check for cached image
+        with lockutils.lock(image_meta.id):
+            vg_wrap = self._get_vg_wrap()
+            cache_name = self.get_name_by_uuid(disk_dvr.DiskType.IMAGE,
+                                               image_meta.id, short=True)
+            image = [disk for disk in vg_wrap.virtual_disks
+                     if disk.name == cache_name]
+            if len(image) == 1:
+                return image[0].udid
+
+            image = tsk_stg.upload_new_vdisk(
+                self.adapter, self._vios_uuid, self.vg_uuid,
+                disk_dvr.IterableToFileAdapter(
+                    IMAGE_API.download(context, image_meta.id)), cache_name,
+                image_meta.size, d_size=image_meta.size,
+                upload_type=tsk_stg.UploadType.IO_STREAM,
+                file_format=image_meta.disk_format)[0]
+            return image.udid
+
+    def connect_disk(self, instance, disk_info, stg_ftsk=None):
         """Connects the disk image to the Virtual Machine.
 
-        :param context: nova context for the transaction.
         :param instance: nova instance to connect the disk to.
         :param disk_info: The pypowervm storage element returned from
                           create_disk_from_image.  Ex. VOptMedia, VDisk, LU,
@@ -229,9 +269,8 @@ class LocalStorage(disk_dvr.DiskAdapter):
                 self.adapter, name='localdisk', xag=[pvm_const.XAG.VIO_SMAP])
 
         def add_func(vios_w):
-            LOG.info(_LI("Adding logical volume disk connection between VM "
-                         "%(vm)s and VIOS %(vios)s."),
-                     {'vm': instance.name, 'vios': vios_w.name})
+            LOG.info("Adding logical volume disk connection to VIOS %(vios)s.",
+                     {'vios': vios_w.name}, instance=instance)
             mapping = tsk_map.build_vscsi_mapping(
                 self.host_uuid, vios_w, lpar_uuid, disk_info)
             return tsk_map.add_map(vios_w, mapping)
@@ -242,10 +281,21 @@ class LocalStorage(disk_dvr.DiskAdapter):
         if stg_ftsk.name == 'localdisk':
             stg_ftsk.execute()
 
-    def extend_disk(self, context, instance, disk_info, size):
+    @staticmethod
+    def _validate_resizable(vdisk):
+        """Validates that VDisk supports resizing
+
+        :param vdisk: The VDisk to be resized
+        :raise ResizeError: If resizing is not supported for the given VDisk.
+        """
+        if vdisk.backstore_type == pvm_stg.BackStoreType.USER_QCOW:
+            raise nova_exc.ResizeError(
+                reason=_("Resizing file-backed instances is not currently "
+                         "supported."))
+
+    def extend_disk(self, instance, disk_info, size):
         """Extends the disk.
 
-        :param context: nova context for operation.
         :param instance: instance to extend the disk for.
         :param disk_info: dictionary with disk info.
         :param size: the new size in gb.
@@ -257,15 +307,17 @@ class LocalStorage(disk_dvr.DiskAdapter):
             vdisks = vg_wrap.virtual_disks
             disk_found = None
             for vdisk in vdisks:
-                if vdisk.name == vol_name:
+                # Vdisk name can be either disk_name or /path/to/disk_name
+                if vdisk.name.split('/')[-1] == vol_name.split('/')[-1]:
                     disk_found = vdisk
                     break
 
             if not disk_found:
-                LOG.error(_LE('Disk %s not found during resize.'), vol_name,
+                LOG.error('Disk %s not found during resize.', vol_name,
                           instance=instance)
                 raise nova_exc.DiskNotFound(
                     location=self.vg_name + '/' + vol_name)
+            self._validate_resizable(disk_found)
 
             # Set the new size
             disk_found.capacity = size
@@ -275,52 +327,16 @@ class LocalStorage(disk_dvr.DiskAdapter):
 
         # Get the disk name based on the instance and type
         vol_name = self._get_disk_name(disk_info['type'], instance, short=True)
-        LOG.info(_LI('Extending disk: %s'), vol_name)
+        LOG.info('Extending disk: %s', vol_name, instance=instance)
         try:
             _extend()
         except pvm_exc.Error:
-            # TODO(IBM): Handle etag mismatch and retry
-            LOG.exception()
-            raise
-
-    def _get_vg_uuid(self, name):
-        """Returns the VIOS and VG UUIDs for the volume group.
-
-        Will iterate over the VIOSes to find the VG with the name.
-
-        :param name: The name of the volume group.
-        :return vios_uuid: The Virtual I/O Server pypowervm UUID.
-        :return vg_uuid: The Volume Group pypowervm UUID.
-        """
-        if CONF.powervm.volume_group_vios_name:
-            # Search for the VIOS if the admin specified it.
-            vios_wraps = pvm_vios.VIOS.search(
-                self.adapter, name=CONF.powervm.volume_group_vios_name)
-        else:
-            vios_resp = self.adapter.read(pvm_ms.System.schema_type,
-                                          root_id=self.host_uuid,
-                                          child_type=pvm_vios.VIOS.schema_type)
-            vios_wraps = pvm_vios.VIOS.wrap(vios_resp)
-
-        # Loop through each vios to find the one with the appropriate name.
-        for vios_wrap in vios_wraps:
-            # Search the feed for the volume group
-            resp = self.adapter.read(pvm_vios.VIOS.schema_type,
-                                     root_id=vios_wrap.uuid,
-                                     child_type=pvm_stg.VG.schema_type)
-            vol_grps = pvm_stg.VG.wrap(resp)
-            for vol_grp in vol_grps:
-                LOG.debug('Volume group: %s', vol_grp.name)
-                if name == vol_grp.name:
-                    return vios_wrap.uuid, vol_grp.uuid
-
-        raise npvmex.VGNotFound(vg_name=name)
-
-    def _get_vg(self):
-        vg_rsp = self.adapter.read(
-            pvm_vios.VIOS.schema_type, root_id=self._vios_uuid,
-            child_type=pvm_stg.VG.schema_type, child_id=self.vg_uuid)
-        return vg_rsp
+            with excutils.save_and_reraise_exception(logger=LOG):
+                # TODO(IBM): Handle etag mismatch and retry
+                LOG.exception("PowerVM Error extending disk.",
+                              instance=instance)
 
     def _get_vg_wrap(self):
-        return pvm_stg.VG.wrap(self._get_vg())
+        return pvm_stg.VG.get(self.adapter, uuid=self.vg_uuid,
+                              parent_type=pvm_vios.VIOS,
+                              parent_uuid=self._vios_uuid)
