@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -14,25 +14,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-
 from pypowervm.tasks import scsi_mapper as pvm_smap
 
 from oslo_log import log as logging
+from taskflow import task
 from taskflow.types import failure as task_fail
 
 from nova_powervm.virt.powervm.disk import driver as disk_driver
 from nova_powervm.virt.powervm import exception as npvmex
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import media
 from nova_powervm.virt.powervm import mgmt
-from nova_powervm.virt.powervm.tasks import base as pvm_task
+
 
 LOG = logging.getLogger(__name__)
 
 
-class ConnectVolume(pvm_task.PowerVMTask):
+class ConnectVolume(task.Task):
 
     """The task to connect a volume to an instance."""
 
@@ -48,20 +45,18 @@ class ConnectVolume(pvm_task.PowerVMTask):
         self.vol_id = self.vol_drv.connection_info['data']['volume_id']
         self.slot_mgr = slot_mgr
 
-        super(ConnectVolume, self).__init__(
-            self.vol_drv.instance, 'connect_vol_%s' % self.vol_id)
+        super(ConnectVolume, self).__init__('connect_vol_%s' % self.vol_id)
 
-    def execute_impl(self):
-        LOG.info(_LI('Connecting volume %(vol)s to instance %(inst)s'),
-                 {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+    def execute(self):
+        LOG.info('Connecting volume %(vol)s.', {'vol': self.vol_id},
+                 instance=self.vol_drv.instance)
         self.vol_drv.connect_volume(self.slot_mgr)
 
-    def revert_impl(self, result, flow_failures):
+    def revert(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
-                        'disconnected'),
-                    {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+        LOG.warning('Rolling back connection for volume %(vol)s.',
+                    {'vol': self.vol_id}, instance=self.vol_drv.instance)
 
         # Note that the rollback is *instant*.  Resetting the FeedTask ensures
         # immediate rollback.
@@ -72,16 +67,14 @@ class ConnectVolume(pvm_task.PowerVMTask):
             # was connected.  This attempts to clear anything out to make sure
             # the terminate connection runs smoothly.
             self.vol_drv.disconnect_volume(self.slot_mgr)
-        except npvmex.VolumeDetachFailed as e:
+        except npvmex.VolumeDetachFailed:
             # Only log that the volume detach failed.  Should not be blocking
             # due to being in the revert flow.
-            LOG.warning(_LW("Unable to disconnect volume for %(inst)s during "
-                            "rollback.  Error was: %(error)s"),
-                        {'inst': self.vol_drv.instance.name,
-                         'error': e.message})
+            LOG.exception("Unable to disconnect volume %s during rollback.",
+                          self.vol_id, instance=self.vol_drv.instance)
 
 
-class DisconnectVolume(pvm_task.PowerVMTask):
+class DisconnectVolume(task.Task):
 
     """The task to disconnect a volume from an instance."""
 
@@ -96,20 +89,20 @@ class DisconnectVolume(pvm_task.PowerVMTask):
         self.vol_drv = vol_drv
         self.vol_id = self.vol_drv.connection_info['data']['volume_id']
         self.slot_mgr = slot_mgr
-        super(DisconnectVolume, self).__init__(
-            self.vol_drv.instance, 'disconnect_vol_%s' % self.vol_id)
 
-    def execute_impl(self):
-        LOG.info(_LI('Disconnecting volume %(vol)s from instance %(inst)s'),
-                 {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+        super(DisconnectVolume, self).__init__(
+            'disconnect_vol_%s' % self.vol_id)
+
+    def execute(self):
+        LOG.info('Disconnecting volume %(vol)s.',
+                 {'vol': self.vol_id}, instance=self.vol_drv.instance)
         self.vol_drv.disconnect_volume(self.slot_mgr)
 
-    def revert_impl(self, result, flow_failures):
+    def revert(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
-                        're-connected'),
-                    {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+        LOG.warning('Reconnecting volume %(vol)s on disconnect rollback.',
+                    {'vol': self.vol_id}, instance=self.vol_drv.instance)
 
         # Note that the rollback is *instant*.  Resetting the FeedTask ensures
         # immediate rollback.
@@ -121,20 +114,18 @@ class DisconnectVolume(pvm_task.PowerVMTask):
             # in error scenarios.  This is simply useful for debug purposes
             # if there is an operational error.
             self.vol_drv.connect_volume(self.slot_mgr)
-        except npvmex.VolumeAttachFailed as e:
+        except npvmex.VolumeAttachFailed:
             # Only log that the volume attach failed.  Should not be blocking
             # due to being in the revert flow.  See comment above.
-            LOG.warning(_LW("Unable to re-connect volume for %(inst)s during "
-                            "rollback.  Error was: %(error)s"),
-                        {'inst': self.vol_drv.instance.name,
-                         'error': e.message})
+            LOG.exception("Unable to reconnect volume %s during rollback.",
+                          self.vol_id, instance=self.vol_drv.instance)
 
 
-class CreateDiskForImg(pvm_task.PowerVMTask):
+class CreateDiskForImg(task.Task):
 
     """The Task to create the disk from an image in the storage."""
 
-    def __init__(self, disk_dvr, context, instance, image_meta, disk_size=0,
+    def __init__(self, disk_dvr, context, instance, image_meta,
                  image_type=disk_driver.DiskType.BOOT):
         """Create the Task.
 
@@ -146,24 +137,22 @@ class CreateDiskForImg(pvm_task.PowerVMTask):
         :param instance: The nova instance.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
-        :param disk_size: The size of disk to create. If the size is smaller
-                          than the image, the image size will be used.
         :param image_type: The image type. See disk/driver.py
         """
         super(CreateDiskForImg, self).__init__(
-            instance, 'crt_disk_from_img', provides='disk_dev_info')
+            'crt_disk_from_img', provides='disk_dev_info')
         self.disk_dvr = disk_dvr
         self.context = context
+        self.instance = instance
         self.image_meta = image_meta
-        self.disk_size = disk_size
         self.image_type = image_type
 
-    def execute_impl(self):
+    def execute(self):
         return self.disk_dvr.create_disk_from_image(
-            self.context, self.instance, self.image_meta, self.disk_size,
+            self.context, self.instance, self.image_meta,
             image_type=self.image_type)
 
-    def revert_impl(self, result, flow_failures):
+    def revert(self, result, flow_failures):
         # If there is no result, or its a direct failure, then there isn't
         # anything to delete.
         if result is None or isinstance(result, task_fail.Failure):
@@ -171,23 +160,20 @@ class CreateDiskForImg(pvm_task.PowerVMTask):
 
         # Run the delete.  The result is a single disk.  Wrap into list
         # as the method works with plural disks.
-        self.disk_dvr.delete_disks(self.context, self.instance, [result])
+        self.disk_dvr.delete_disks([result])
 
 
-class ConnectDisk(pvm_task.PowerVMTask):
+class ConnectDisk(task.Task):
 
     """The task to connect the disk to the instance."""
 
-    def __init__(self, disk_dvr, context, instance, stg_ftsk=None):
+    def __init__(self, disk_dvr, instance, stg_ftsk=None):
         """Create the Task for the connect disk to instance method.
 
-        Requires LPAR info through requirement of lpar_wrap.
-
         Requires disk info through requirement of disk_dev_info (provided by
         crt_disk_from_img)
 
         :param disk_dvr: The disk driver.
-        :param context: The context passed into the spawn method.
         :param instance: The nova instance.
         :param stg_ftsk: (Optional) The pypowervm transaction FeedTask for the
                          I/O Operations.  If provided, the Virtual I/O Server
@@ -196,22 +182,22 @@ class ConnectDisk(pvm_task.PowerVMTask):
                          the FeedTask is not provided, the updates will be run
                          immediately when the respective method is executed.
         """
-        super(ConnectDisk, self).__init__(
-            instance, 'connect_disk', requires=['disk_dev_info'])
+        super(ConnectDisk, self).__init__('connect_disk',
+                                          requires=['disk_dev_info'])
         self.disk_dvr = disk_dvr
-        self.context = context
+        self.instance = instance
         self.stg_ftsk = stg_ftsk
 
-    def execute_impl(self, disk_dev_info,):
-        self.disk_dvr.connect_disk(self.context, self.instance, disk_dev_info,
+    def execute(self, disk_dev_info):
+        self.disk_dvr.connect_disk(self.instance, disk_dev_info,
                                    stg_ftsk=self.stg_ftsk)
 
-    def revert_impl(self, disk_dev_info, result, flow_failures):
+    def revert(self, disk_dev_info, result, flow_failures):
         # Note that the FeedTask is None - to force instant disconnect.
-        self.disk_dvr.disconnect_image_disk(self.context, self.instance)
+        self.disk_dvr.disconnect_disk(self.instance)
 
 
-class InstanceDiskToMgmt(pvm_task.PowerVMTask):
+class InstanceDiskToMgmt(task.Task):
 
     """Connect an instance's disk to the management partition, discover it.
 
@@ -235,14 +221,15 @@ class InstanceDiskToMgmt(pvm_task.PowerVMTask):
         :param instance: The nova instance whose boot disk is to be connected.
         """
         super(InstanceDiskToMgmt, self).__init__(
-            instance, 'connect_and_discover_instance_disk_to_mgmt',
+            'connect_and_discover_instance_disk_to_mgmt',
             provides=['stg_elem', 'vios_wrap', 'disk_path'])
         self.disk_dvr = disk_dvr
+        self.instance = instance
         self.stg_elem = None
         self.vios_wrap = None
         self.disk_path = None
 
-    def execute_impl(self):
+    def execute(self):
         """Map the instance's boot disk and discover it."""
 
         # Search for boot disk on the Novalink partition
@@ -266,13 +253,13 @@ class InstanceDiskToMgmt(pvm_task.PowerVMTask):
         # partition from the same VIOS - it is safe to use the first one.
         the_map = new_maps[0]
         # Scan the SCSI bus, discover the disk, find its canonical path.
-        LOG.info(_LI("Discovering device and path for mapping of %(dev_name)s "
-                     "on the management partition."),
-                 {'dev_name': self.stg_elem.name})
+        LOG.info("Discovering device and path for mapping of %(dev_name)s "
+                 "on the management partition.",
+                 {'dev_name': self.stg_elem.name}, instance=self.instance)
         self.disk_path = mgmt.discover_vscsi_disk(the_map)
         return self.stg_elem, self.vios_wrap, self.disk_path
 
-    def revert_impl(self, result, flow_failures):
+    def revert(self, result, flow_failures):
         """Unmap the disk and then remove it from the management partition.
 
         We use this order to avoid rediscovering the device in case some other
@@ -281,24 +268,22 @@ class InstanceDiskToMgmt(pvm_task.PowerVMTask):
         if self.vios_wrap is None or self.stg_elem is None:
             # We never even got connected - nothing to do
             return
-        LOG.warning(_LW("Unmapping boot disk %(disk_name)s of instance "
-                        "%(instance_name)s from management partition via "
-                        "Virtual I/O Server %(vios_name)s."),
+        LOG.warning("Unmapping boot disk %(disk_name)s from the management "
+                    "partition via Virtual I/O Server %(vioname)s.",
                     {'disk_name': self.stg_elem.name,
-                     'instance_name': self.instance.name,
-                     'vios_name': self.vios_wrap.name})
+                     'vioname': self.vios_wrap.name}, instance=self.instance)
         self.disk_dvr.disconnect_disk_from_mgmt(self.vios_wrap.uuid,
                                                 self.stg_elem.name)
 
         if self.disk_path is None:
             # We did not discover the disk - nothing else to do.
             return
-        LOG.warning(_LW("Removing disk %(disk_path)s from the management "
-                        "partition."), {'disk_path': self.disk_path})
+        LOG.warning("Removing disk %(dpath)s from the management partition.",
+                    {'dpath': self.disk_path}, instance=self.instance)
         mgmt.remove_block_dev(self.disk_path)
 
 
-class RemoveInstanceDiskFromMgmt(pvm_task.PowerVMTask):
+class RemoveInstanceDiskFromMgmt(task.Task):
 
     """Unmap and remove an instance's boot disk from the mgmt partition."""
 
@@ -318,11 +303,12 @@ class RemoveInstanceDiskFromMgmt(pvm_task.PowerVMTask):
         :param instance: The nova instance whose boot disk is to be connected.
         """
         self.disk_dvr = disk_dvr
+        self.instance = instance
         super(RemoveInstanceDiskFromMgmt, self).__init__(
-            instance, 'remove_inst_disk_from_mgmt',
+            'remove_inst_disk_from_mgmt',
             requires=['stg_elem', 'vios_wrap', 'disk_path'])
 
-    def execute_impl(self, stg_elem, vios_wrap, disk_path):
+    def execute(self, stg_elem, vios_wrap, disk_path):
         """Unmap and remove an instance's boot disk from the mgmt partition.
 
         Input parameters ('requires') provided by InstanceDiskToMgmt task.
@@ -337,19 +323,17 @@ class RemoveInstanceDiskFromMgmt(pvm_task.PowerVMTask):
         # stg_elem is None if boot disk was not mapped to management partition
         if stg_elem is None:
             return
-        LOG.info(_LI("Unmapping boot disk %(disk_name)s of instance "
-                     "%(instance_name)s from management partition via Virtual "
-                     "I/O Server %(vios_name)s."),
-                 {'disk_name': stg_elem.name,
-                  'instance_name': self.instance.name,
-                  'vios_name': vios_wrap.name})
+        LOG.info("Unmapping boot disk %(disk_name)s from the management "
+                 "partition via Virtual I/O Server %(vios_name)s.",
+                 {'disk_name': stg_elem.name, 'vios_name': vios_wrap.name},
+                 instance=self.instance)
         self.disk_dvr.disconnect_disk_from_mgmt(vios_wrap.uuid, stg_elem.name)
-        LOG.info(_LI("Removing disk %(disk_path)s from the management "
-                     "partition."), {'disk_path': disk_path})
+        LOG.info("Removing disk %(disk_path)s from the management partition.",
+                 {'disk_path': disk_path}, instance=self.instance)
         mgmt.remove_block_dev(disk_path)
 
 
-class CreateAndConnectCfgDrive(pvm_task.PowerVMTask):
+class CreateAndConnectCfgDrive(task.Task):
 
     """The task to create the configuration drive."""
 
@@ -376,23 +360,24 @@ class CreateAndConnectCfgDrive(pvm_task.PowerVMTask):
                          immediately when the respective method is executed.
         """
         super(CreateAndConnectCfgDrive, self).__init__(
-            instance, 'cfg_drive', requires=['lpar_wrap', 'mgmt_cna'])
+            'cfg_drive', requires=['lpar_wrap', 'mgmt_cna'])
         self.adapter = adapter
         self.host_uuid = host_uuid
+        self.instance = instance
         self.injected_files = injected_files
         self.network_info = network_info
         self.ad_pass = admin_pass
         self.mb = None
         self.stg_ftsk = stg_ftsk
 
-    def execute_impl(self, lpar_wrap, mgmt_cna):
+    def execute(self, lpar_wrap, mgmt_cna):
         self.mb = media.ConfigDrivePowerVM(self.adapter, self.host_uuid)
         self.mb.create_cfg_drv_vopt(self.instance, self.injected_files,
                                     self.network_info, lpar_wrap.uuid,
                                     admin_pass=self.ad_pass,
                                     mgmt_cna=mgmt_cna, stg_ftsk=self.stg_ftsk)
 
-    def revert_impl(self, lpar_wrap, mgmt_cna, result, flow_failures):
+    def revert(self, lpar_wrap, mgmt_cna, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
 
@@ -403,13 +388,12 @@ class CreateAndConnectCfgDrive(pvm_task.PowerVMTask):
         # Delete the virtual optical media. If it fails we don't care.
         try:
             self.mb.dlt_vopt(lpar_wrap.uuid)
-        except Exception as e:
-            LOG.warning(_LW('Vopt removal as part of spawn reversion failed '
-                            'with: %(exc)s'), {'exc': six.text_type(e)},
-                        instance=self.instance)
+        except Exception:
+            LOG.exception('VOpt removal (as part of reversion) failed.',
+                          instance=self.instance)
 
 
-class DeleteVOpt(pvm_task.PowerVMTask):
+class DeleteVOpt(task.Task):
 
     """The task to delete the virtual optical."""
 
@@ -428,30 +412,29 @@ class DeleteVOpt(pvm_task.PowerVMTask):
                          the FeedTask is not provided, the updates will be run
                          immediately when the respective method is executed.
         """
-        super(DeleteVOpt, self).__init__(instance, 'vopt_delete')
+        super(DeleteVOpt, self).__init__('vopt_delete')
         self.adapter = adapter
         self.host_uuid = host_uuid
+        self.instance = instance
         self.lpar_uuid = lpar_uuid
         self.stg_ftsk = stg_ftsk
 
-    def execute_impl(self):
+    def execute(self):
         media_builder = media.ConfigDrivePowerVM(self.adapter, self.host_uuid)
         media_builder.dlt_vopt(self.lpar_uuid, stg_ftsk=self.stg_ftsk)
 
 
-class DetachDisk(pvm_task.PowerVMTask):
+class DetachDisk(task.Task):
 
     """The task to detach the disk storage from the instance."""
 
-    def __init__(self, disk_dvr, context, instance, stg_ftsk=None,
-                 disk_type=None):
+    def __init__(self, disk_dvr, instance, stg_ftsk=None, disk_type=None):
         """Creates the Task to detach the storage adapters.
 
         Provides the stor_adpt_mappings.  A list of pypowervm
         VSCSIMappings or VFCMappings (depending on the storage adapter).
 
         :param disk_dvr: The DiskAdapter for the VM.
-        :param context: The nova context.
         :param instance: The nova instance.
         :param stg_ftsk: (Optional) The pypowervm transaction FeedTask for the
                          I/O Operations.  If provided, the Virtual I/O Server
@@ -462,42 +445,38 @@ class DetachDisk(pvm_task.PowerVMTask):
         :param disk_type: List of disk types to detach. None means detach all.
         """
         super(DetachDisk, self).__init__(
-            instance, 'detach_storage', provides='stor_adpt_mappings')
+            'detach_storage', provides='stor_adpt_mappings')
         self.disk_dvr = disk_dvr
-        self.context = context
+        self.instance = instance
         self.stg_ftsk = stg_ftsk
         self.disk_type = disk_type
 
-    def execute_impl(self):
-        return self.disk_dvr.disconnect_image_disk(
-            self.context, self.instance, stg_ftsk=self.stg_ftsk,
-            disk_type=self.disk_type)
+    def execute(self):
+        return self.disk_dvr.disconnect_disk(
+            self.instance, stg_ftsk=self.stg_ftsk, disk_type=self.disk_type)
 
 
-class DeleteDisk(pvm_task.PowerVMTask):
+class DeleteDisk(task.Task):
 
     """The task to delete the backing storage."""
 
-    def __init__(self, disk_dvr, context, instance):
+    def __init__(self, disk_dvr, instance):
         """Creates the Task to delete the disk storage from the system.
 
         Requires the stor_adpt_mappings.
 
         :param disk_dvr: The DiskAdapter for the VM.
-        :param context: The nova context.
         :param instance: The nova instance.
         """
         req = ['stor_adpt_mappings']
-        super(DeleteDisk, self).__init__(instance, 'dlt_storage', requires=req)
+        super(DeleteDisk, self).__init__('dlt_storage', requires=req)
         self.disk_dvr = disk_dvr
-        self.context = context
 
-    def execute_impl(self, stor_adpt_mappings):
-        self.disk_dvr.delete_disks(self.context, self.instance,
-                                   stor_adpt_mappings)
+    def execute(self, stor_adpt_mappings):
+        self.disk_dvr.delete_disks(stor_adpt_mappings)
 
 
-class SaveBDM(pvm_task.PowerVMTask):
+class SaveBDM(task.Task):
 
     """Task to save an updated block device mapping."""
 
@@ -508,17 +487,16 @@ class SaveBDM(pvm_task.PowerVMTask):
         :param instance: The nova instance
         """
         self.bdm = bdm
-        super(SaveBDM, self).__init__(
-            instance, 'save_bdm_%s' % self.bdm.volume_id)
+        self.instance = instance
+        super(SaveBDM, self).__init__('save_bdm_%s' % self.bdm.volume_id)
 
-    def execute_impl(self):
-        LOG.info(_LI('Saving block device mapping for volume id %(vol_id)s '
-                     'on instance %(inst)s.'),
-                 {'vol_id': self.bdm.volume_id, 'inst': self.instance.name})
+    def execute(self):
+        LOG.info('Saving block device mapping for volume id %(vol_id)s.',
+                 {'vol_id': self.bdm.volume_id}, instance=self.instance)
         self.bdm.save()
 
 
-class FindDisk(pvm_task.PowerVMTask):
+class FindDisk(task.Task):
 
     """The Task to find a disk and provide information to downstream tasks."""
 
@@ -533,16 +511,16 @@ class FindDisk(pvm_task.PowerVMTask):
         :param instance: The nova instance.
         :param disk_type: One of the DiskType enum values.
         """
-        super(FindDisk, self).__init__(
-            instance, 'find_disk', provides='disk_dev_info')
+        super(FindDisk, self).__init__('find_disk', provides='disk_dev_info')
         self.disk_dvr = disk_dvr
         self.context = context
+        self.instance = instance
         self.disk_type = disk_type
 
-    def execute_impl(self):
+    def execute(self):
         disk = self.disk_dvr.get_disk_ref(self.instance, self.disk_type)
         if not disk:
-            LOG.warning(_LW('Disk not found: %(disk_name)s'),
+            LOG.warning('Disk not found: %(disk_name)s',
                         {'disk_name':
                             self.disk_dvr._get_disk_name(self.disk_type,
                                                          self.instance),
@@ -550,28 +528,26 @@ class FindDisk(pvm_task.PowerVMTask):
         return disk
 
 
-class ExtendDisk(pvm_task.PowerVMTask):
+class ExtendDisk(task.Task):
 
     """Task to extend a disk."""
 
-    def __init__(self, disk_dvr, context, instance, disk_info, size):
+    def __init__(self, disk_dvr, instance, disk_info, size):
         """Creates the Task to extend a disk.
 
         :param disk_dvr: The storage driver.
-        :param context: nova context for operation.
         :param instance: instance to extend the disk for.
         :param disk_info: dictionary with disk info.
         :param size: the new size in gb.
         """
         self.disk_dvr = disk_dvr
-        self.context = context
+        self.instance = instance
         self.disk_info = disk_info
         self.size = size
-        super(ExtendDisk, self).__init__(
-            instance, 'extend_disk_%s' % disk_info['type'])
+        super(ExtendDisk, self).__init__('extend_disk_%s' % disk_info['type'])
 
-    def execute_impl(self):
-        LOG.info(_LI('Extending disk size of disk: %(disk)s size: %(size)s.'),
-                 {'disk': self.disk_info['type'], 'size': self.size})
-        self.disk_dvr.extend_disk(self.context, self.instance, self.disk_info,
-                                  self.size)
+    def execute(self):
+        LOG.info('Extending %(disk_type)s disk to %(size)s GB.',
+                 {'disk_type': self.disk_info['type'], 'size': self.size},
+                 instance=self.instance)
+        self.disk_dvr.extend_disk(self.instance, self.disk_info, self.size)
