@@ -1,4 +1,4 @@
-# Copyright 2014, 2015, 2016 IBM Corp.
+# Copyright 2014, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import re
@@ -28,6 +29,7 @@ from pypowervm import exceptions as pvm_exc
 from pypowervm.helpers import log_helper as pvm_log
 from pypowervm.tasks import ibmi
 from pypowervm.tasks import power
+from pypowervm.tasks import power_opts as popts
 from pypowervm.tasks import vterm
 from pypowervm import util as pvm_util
 from pypowervm.utils import lpar_builder as lpar_bldr
@@ -43,9 +45,7 @@ from pypowervm.wrappers import shared_proc_pool as pvm_spp
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as nvex
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
+
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -125,7 +125,8 @@ def translate_event(pvm_state, pwr_state):
     elif pvm_state in RESUMING_EVENTS and pwr_state != power_state.RUNNING:
         trans = event.EVENT_LIFECYCLE_RESUMED
 
-    LOG.debug('Translated Event to %s', trans)
+    LOG.debug('Translated {PowerVM state %s; power state %s} to %s',
+              pvm_state, pwr_state, trans)
     return trans
 
 
@@ -279,13 +280,12 @@ class VMBuilder(object):
             kwargs['max_slots'] = slot_mgr.build_map.get_max_vslots()
         self.stdz = lpar_bldr.DefaultStandardize(self.host_w, **kwargs)
 
-    def lpar_builder(self, instance, flavor):
+    def lpar_builder(self, instance):
         """Returns the pypowervm LPARBuilder for a given Nova flavor.
 
         :param instance: the VM instance
-        :param flavor: The Nova instance flavor.
         """
-        attrs = self._format_flavor(instance, flavor)
+        attrs = self._format_flavor(instance)
         self._add_IBMi_attrs(instance, attrs)
         return lpar_bldr.LPARBuilder(self.adapter, attrs, self.stdz)
 
@@ -295,11 +295,10 @@ class VMBuilder(object):
             attrs[lpar_bldr.ENV] = pvm_bp.LPARType.OS400
             # Add other attributes in the future
 
-    def _format_flavor(self, instance, flavor):
+    def _format_flavor(self, instance):
         """Returns the pypowervm format of the flavor.
 
         :param instance: the VM instance
-        :param flavor: The Nova instance flavor.
         :return: a dict that can be used by the LPAR builder
         """
         # The attrs are what is sent to pypowervm to convert the lpar.
@@ -309,13 +308,13 @@ class VMBuilder(object):
             instance.name)
         # The uuid is only actually set on a create of an LPAR
         attrs[lpar_bldr.UUID] = pvm_uuid.convert_uuid_to_pvm(instance.uuid)
-        attrs[lpar_bldr.MEM] = flavor.memory_mb
-        attrs[lpar_bldr.VCPU] = flavor.vcpus
+        attrs[lpar_bldr.MEM] = instance.flavor.memory_mb
+        attrs[lpar_bldr.VCPU] = instance.flavor.vcpus
         # Set the srr capability to True by default
         attrs[lpar_bldr.SRR_CAPABLE] = True
 
         # Loop through the extra specs and process powervm keys
-        for key in flavor.extra_specs.keys():
+        for key in instance.flavor.extra_specs.keys():
             # If it is not a valid key, then can skip.
             if not self._is_pvm_valid_key(key):
                 continue
@@ -326,10 +325,10 @@ class VMBuilder(object):
             # Check for no direct mapping, if the value is none, need to
             # derive the complex type
             if bldr_key is None:
-                self._build_complex_type(key, attrs, flavor)
+                self._build_complex_type(key, attrs, instance.flavor)
             else:
                 # We found a direct mapping
-                attrs[bldr_key] = flavor.extra_specs[key]
+                attrs[bldr_key] = instance.flavor.extra_specs[key]
 
         return attrs
 
@@ -379,7 +378,7 @@ class VMBuilder(object):
             else:
                 attr = key + '=' + flavor.extra_specs[key]
                 exc = exception.InvalidAttribute(attr=attr)
-                LOG.exception(exc)
+                LOG.error(exc)
                 raise exc
         elif key == self._PVM_SHAR_PROC_POOL:
             pool_name = flavor.extra_specs[key]
@@ -394,7 +393,7 @@ class VMBuilder(object):
         else:
             # There was no mapping or we didn't handle it.
             exc = exception.InvalidAttribute(attr=key)
-            LOG.exception(exc)
+            LOG.error(exc)
             raise exc
 
     def _spp_pool_id(self, pool_name):
@@ -467,14 +466,8 @@ def get_instance_wrapper(adapter, instance, xag=None):
     pvm_inst_uuid = get_pvm_uuid(instance)
     try:
         return pvm_lpar.LPAR.get(adapter, uuid=pvm_inst_uuid, xag=xag)
-    except pvm_exc.Error as he:
-        if he.response is not None and he.response.status == 404:
-            LOG.exception(he)
-            # Raise InstanceNotFound exception
-            raise exception.InstanceNotFound(instance_id=pvm_inst_uuid)
-        else:
-            LOG.exception(he)
-            raise
+    except pvm_exc.HttpNotFound:
+        raise exception.InstanceNotFound(instance_id=pvm_inst_uuid)
 
 
 def instance_exists(adapter, instance, log_errors=False):
@@ -529,25 +522,20 @@ def get_vm_qp(adapter, lpar_uuid, qprop=None, log_errors=True):
                 pass
             kwds['helpers'] = helpers
         resp = adapter.read(pvm_lpar.LPAR.schema_type, **kwds)
-    except pvm_exc.HttpError as e:
+    except pvm_exc.HttpNotFound:
         # 404 error indicates the LPAR has been deleted (or moved to a
         # different host)
-        if e.response and e.response.status == 404:
-            raise exception.InstanceNotFound(instance_id=lpar_uuid)
-        else:
-            raise
+        raise exception.InstanceNotFound(instance_id=lpar_uuid)
 
     return jsonutils.loads(resp.body)
 
 
-def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None,
-             slot_mgr=None):
+def create_lpar(adapter, host_wrapper, instance, nvram=None, slot_mgr=None):
     """Create an LPAR based on the host based on the instance
 
     :param adapter: The adapter for the pypowervm API
     :param host_wrapper: The host wrapper
     :param instance: The nova instance.
-    :param flavor: The nova flavor.
     :param nvram: The NVRAM to set on the LPAR.
     :param slot_mgr: NovaSlotManager to restore/save the maximum number of
                      virtual slots.  If omitted, the default is used.
@@ -555,8 +543,7 @@ def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None,
     """
     try:
         lpar_b = VMBuilder(
-            host_wrapper, adapter, slot_mgr=slot_mgr).lpar_builder(instance,
-                                                                   flavor)
+            host_wrapper, adapter, slot_mgr=slot_mgr).lpar_builder(instance)
         pending_lpar_w = lpar_b.build()
         vldn.LPARWrapperValidator(pending_lpar_w, host_wrapper).validate_all()
         if nvram is not None:
@@ -571,17 +558,16 @@ def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None,
                                             reason=e)
     except pvm_exc.HttpError as he:
         # Raise the API exception
-        LOG.exception(he)
+        LOG.exception("PowerVM HttpError creating LPAR.", instance=instance)
         raise nvex.PowerVMAPIFailed(inst_name=instance.name, reason=he)
 
 
-def update(adapter, host_wrapper, instance, flavor, entry=None, name=None):
+def update(adapter, host_wrapper, instance, entry=None, name=None):
     """Update an LPAR based on the host based on the instance
 
     :param adapter: The adapter for the pypowervm API
     :param host_wrapper: The host wrapper
     :param instance: The nova instance.
-    :param flavor: The nova flavor.
     :param entry: The instance pvm entry, if available, otherwise it will
         be fetched.
     :param name: VM name to use for the update.  Used on resize when we want
@@ -592,7 +578,7 @@ def update(adapter, host_wrapper, instance, flavor, entry=None, name=None):
     if not entry:
         entry = get_instance_wrapper(adapter, instance)
 
-    lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance, flavor)
+    lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance)
     lpar_b.rebuild(entry)
 
     # Set the new name if the instance name is not desired.
@@ -625,100 +611,126 @@ def rename(adapter, instance, name, entry=None):
     return _rename(entry)
 
 
-def dlt_lpar(adapter, lpar_uuid):
+def delete_lpar(adapter, instance):
     """Delete an LPAR
 
-    :param adapter: The adapter for the pypowervm API
-    :param lpar_uuid: The lpar to delete
+    :param adapter: The adapter for the pypowervm API.
+    :param instance: The nova instance whose LPAR is to be deleted.
     """
+    lpar_uuid = get_pvm_uuid(instance)
     # Attempt to delete the VM.
     try:
-        LOG.info(_LI('Deleting virtual machine. LPARID: %s'), lpar_uuid)
+        LOG.info('Deleting LPAR', instance=instance)
 
         # Ensure any vterms are closed.  Will no-op otherwise.
         vterm.close_vterm(adapter, lpar_uuid)
 
         # Run the LPAR delete
         resp = adapter.delete(pvm_lpar.LPAR.schema_type, root_id=lpar_uuid)
-        LOG.info(_LI('Virtual machine delete status: %d'), resp.status)
+        LOG.info('LPAR delete status: %d', resp.status, instance=instance)
         return resp
-    except pvm_exc.HttpError as e:
-        if e.response and e.response.status == 404:
-            LOG.warning(_LW('Virtual Machine not found LPAR_ID: %s'),
-                        lpar_uuid)
-        else:
-            LOG.error(_LE('HttpError deleting virtual machine. LPARID: %s'),
-                      lpar_uuid)
-            raise
-    except pvm_exc.Error:
-        # Attempting to close vterm did not help so raise exception
-        LOG.error(_LE('Virtual machine delete failed: LPARID=%s'),
-                  lpar_uuid)
-        raise
+    except pvm_exc.HttpNotFound:
+        LOG.info('LPAR not found (already deleted).', instance=instance)
 
 
-def power_on(adapter, instance, host_uuid, entry=None):
-    if entry is None:
+def power_on(adapter, instance, opts=None):
+    """Powers on a VM.
+
+    :param adapter: A pypowervm.adapter.Adapter.
+    :param instance: The nova instance to power on.
+    :param opts: (Optional) Additional parameters to the pypowervm power_on
+                 method.  See that method's docstring for details.
+    :return: True if the instance was powered on.  False if it was not in a
+             startable state.
+    :raises: InstancePowerOnFailure
+    """
+    # Synchronize power-on and power-off ops on a given instance
+    with lockutils.lock('power_%s' % instance.uuid):
         entry = get_instance_wrapper(adapter, instance)
 
-    # Get the current state and see if we can start the VM
-    if entry.state in POWERVM_STARTABLE_STATE:
-        # Now start the lpar
-        power.power_on(entry, host_uuid)
-        return True
+        # Get the current state and see if we can start the VM
+        if entry.state in POWERVM_STARTABLE_STATE:
+            # Now start the lpar
+            power.power_on(entry, None, add_parms=opts)
+            return True
 
     return False
 
 
-def power_off(adapter, instance, host_uuid, entry=None, add_parms=None,
-              force_immediate=False, timeout=None):
-    """Powers Off a VM.
+def power_off(adapter, instance, force_immediate=False, timeout=None):
+    """Powers off a VM.
 
-    :param instance: The nova instance.
-    :param host_uuid: The uuid of the host.
-    :param entry: (Optional) The pypowervm wrapper for the entry.  If not
-                  provided, will be looked up.
-    :param add_parms: (Optional) Additional parameters to the pypowervm
-                      power_off method.  See that method's docstring for
-                      details.
+    :param adapter: A pypowervm.adapter.Adapter.
+    :param instance: The nova instance to power off.
     :param force_immediate: (Optional, Default False) Should it be immediately
                             shut down.
     :param timeout: (Optional, Default None) How long to wait for the job
                     to complete.  By default, is None which indicates it should
                     use the default from pypowervm's power off method.
-    :return: True if it was stopped.  False if it was not in a stoppable state.
+    :return: True if the instance was stopped.  False if it was not in a
+             stoppable state.
     :raises: InstancePowerOffFailure
     """
-    if entry is None:
+    # Synchronize power-on and power-off ops on a given instance
+    with lockutils.lock('power_%s' % instance.uuid):
         entry = get_instance_wrapper(adapter, instance)
 
-    # Get the current state and see if we can stop the VM
-    LOG.debug("Powering off request for instance %(inst)s which is in "
-              "state %(state)s.  Force Immediate Flag: %(force)s.",
-              {'inst': instance.name, 'state': entry.state,
-               'force': force_immediate})
-    if entry.state in POWERVM_STOPABLE_STATE:
-        # Now stop the lpar
-        try:
-            LOG.debug("Power off executing for instance %(inst)s.",
-                      {'inst': instance.name})
-            kwargs = {'timeout': timeout} if timeout else {}
-            force_flag = (power.Force.TRUE if force_immediate
-                          else power.Force.ON_FAILURE)
-            power.power_off(
-                entry, host_uuid, force_immediate=force_flag,
-                add_parms=add_parms, **kwargs)
-        except Exception as e:
-            LOG.exception(e)
-            raise exception.InstancePowerOffFailure(reason=six.text_type(e))
-        return True
-    else:
-        LOG.debug("Power off not required for instance %(inst)s.",
-                  {'inst': instance.name})
+        # Get the current state and see if we can stop the VM
+        LOG.debug("Power off requested for instance in state %(state)s. Force "
+                  "Immediate Flag: %(force)s.",
+                  {'state': entry.state, 'force': force_immediate},
+                  instance=instance)
+        if entry.state in POWERVM_STOPABLE_STATE:
+            # Now stop the lpar
+            try:
+                LOG.debug("Power off executing.", instance=instance)
+                kwargs = {'timeout': timeout} if timeout else {}
+                if force_immediate:
+                    power.PowerOp.stop(
+                        entry, opts=popts.PowerOffOpts().vsp_hard(), **kwargs)
+                else:
+                    power.power_off_progressive(entry, **kwargs)
+            except Exception as e:
+                LOG.exception("Failed to power off instance.",
+                              instance=instance)
+                raise exception.InstancePowerOffFailure(
+                    reason=six.text_type(e))
+            return True
+        else:
+            LOG.debug("Power off not required.", instance=instance)
 
     return False
 
 
+def reboot(adapter, instance, hard):
+    """Reboots a VM.
+
+    :param adapter: A pypowervm.adapter.Adapter.
+    :param instance: The nova instance to reboot.
+    :param hard: Boolean True if hard reboot, False otherwise.
+    :raises: InstanceRebootFailure
+    """
+    # Synchronize power-on and power-off ops on a given instance
+    with lockutils.lock('power_%s' % instance.uuid):
+        try:
+            entry = get_instance_wrapper(adapter, instance)
+            if entry.state != pvm_bp.LPARState.NOT_ACTIVATED:
+                if hard:
+                    power.PowerOp.stop(
+                        entry, opts=popts.PowerOffOpts().vsp_hard().restart())
+                else:
+                    power.power_off_progressive(entry, restart=True)
+            else:
+                # pypowervm does NOT throw an exception if "already down".
+                # Any other exception from pypowervm is a legitimate failure;
+                # let it raise up.
+                # If we get here, pypowervm thinks the instance is down.
+                power.power_on(entry, None)
+        except Exception as e:
+            LOG.exception("Failed to reboot instance.", instance=instance)
+            raise exception.InstanceRebootFailure(reason=six.text_type(e))
+
+
 def get_pvm_uuid(instance):
     """Get the corresponding PowerVM VM uuid of an instance uuid
 
@@ -768,7 +780,7 @@ def get_instance(context, pvm_uuid):
     except exception.InstanceNotFound:
         pass
     except Exception as e:
-        LOG.debug('PowerVM UUID not found. %s', e)
+        LOG.debug('Instance with PowerVM UUID %s not found: %s', pvm_uuid, e)
     return None
 
 
@@ -780,7 +792,8 @@ def get_cnas(adapter, instance, **search):
     :param instance: The nova instance.
     :param search: Keyword arguments for CNA.search.  If omitted, all CNAs are
                    returned.
-    :return The CNA wrappers that represent the ClientNetworkAdapters on the VM
+    :return: The CNA wrappers that represent the ClientNetworkAdapters
+    on the VM
     """
     meth = pvm_net.CNA.search if search else pvm_net.CNA.get
 
@@ -795,7 +808,7 @@ def get_vnics(adapter, instance, **search):
     :param instance: The nova instance.
     :param search: Keyword arguments for VNIC.search.  If omitted, all VNICs
                    are returned.
-    :return The VNIC wrappers that represent the virtual NICs on the VM.
+    :return: The VNIC wrappers that represent the virtual NICs on the VM.
     """
     meth = pvm_card.VNIC.search if search else pvm_card.VNIC.get
 
